;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
; Load kernel using UnReal Mode
;-----------------------------------------------------------------------------------


DriveNumber     db 0x00
StartKERNEL64   dd 1+39+64		; first LBA sector
NumChunks	dd 128			; 512 Kib kernel size
NumSectors      dd 8			; 4 KiB chunk
KernelAddress   dd 400000h		; 4' MiB 

;----------------------------------------


;-------------
UM_Switch_To:

        pushad
        push ds
        push ds

	cli					; disable the interrupts
						
	lgdt	[cs:GDTR]			; load GDT register
	mov	eax, cr0 			; switch to protected mode
	or	al, 1
	mov	cr0, eax

	xor	ebx, ebx
	mov	ebx, DATA_SELECTOR		; load 4 GB data descriptor
	mov	ds, bx				; to all data segment registers
	mov	es, bx

	and  	al, 0xFE           		; back to realmode
   	mov 	cr0, eax          		; by toggling bit again

	sti

        pop es                  		; segment regs back to old values,
        pop ds                   		; but now 32-bit addresses are OK
        popad        

	ret

;--------------

UM_Load_Kernel:

	call UM_Switch_To

chunk:
	xor eax, eax
	xor ecx, ecx
	mov bx,  6000h				; Temporary Buffer Address
	mov eax, [StartKERNEL64]
	mov ecx, [NumSectors]

readsec:
	call readsector
	dec ecx
	cmp ecx, 0
	jne readsec


 ; moving 512*8 bytes from low memory to high memory
   	mov ecx, 128*8
	mov esi, 6000h				; Temporary Buffer Address
	mov edi, [KernelAddress]	
  movesectors:
   	mov eax, dword[esi] 
   	mov dword[edi], eax
   	add edi, 4
   	add esi, 4
  loop movesectors


	mov ecx, [NumChunks]
	call UM_progress_bar			; Draw a progress bar
	dec ecx
	cmp ecx, 0
	je end_lk

	dec [NumChunks]
	add [KernelAddress], 4096
	add [StartKERNEL64], 8
	jmp chunk

end_lk:

	ret

;--------------------------------
; IN: ecx = Countdown (NumChunks)
UM_progress_bar:

	pushad
	push es

	mov al, [GUI]
	cmp al, 0
	je  .noGUI

	mov edi, dword[Vesa_modeinfo.LFB]
	xor esi, esi
	mov si, [Vesa_modeinfo.linbytesperline]

	movzx eax, [Vesa_modeinfo.yres]
	shr eax, 1
	sub eax, 128				; NumChunks
	add ecx, eax

	.startline:
	add edi, esi				; start of next line in LFB
	loop .startline

	movzx ecx, [Vesa_modeinfo.xres]
	shr ecx, 1
	sub ecx, 25
	shl ecx, 2				; 4 bytes x pixel
	add edi, ecx

	;--------------------------------------------------------
	mov ecx, 50
	mov eax, 0
	.line:					; a   r   g   b
	mov dword[edi+eax], 0x0000ff00		; 000 000 255 000
	add eax, 4
	loop .line
	;--------------------------------------------------------

.noGUI:
	pop es
	popad
	ret


;-----------------------------------------------------------------------------
Dap:
	dap_sizepacket 	db 16
	dap_reserved   	db 0
	dap_numsec	dw 1
	dap_bufoff     	dw 0		; seg:off little endian
	dap_bufseg  	dw 0		; 
	dap_startlba1	dd 0		; sector number low 32bit
	dap_startlba2	dd 0		; sector number high 32bit

;------------------------------------------------------------------------------
; Read a sector from a disk, using LBA
; input:	EAX - 32-bit DOS sector number
;		ES:BX - destination buffer
; output:	ES:BX points one byte after the last byte read
;		EAX - next sector
readsector:
	push ecx
	push dx
	push si
	push di

read_it:

	push eax		; Save the sector number

	mov [dap_bufoff], bx
	mov [dap_bufseg], es
	mov [dap_startlba1], eax
	mov si, Dap

	mov dl, [DriveNumber]
	mov ah, 42h		; Extended Read
	int 0x13		; 

	pop eax			; Restore the sector number

	jnc read_ok 		; jump if no error

	push ax
	xor ah, ah		; else, reset and retry
	int 0x13
	pop ax
	jmp read_it

read_ok:
	inc eax 		; next sector
	add bx, 512		; Add bytes per sector
	jnc no_incr_es		; if overflow...

incr_es:
	mov dx, es
	add dh, 0x10		; ...add 1000h to ES
	mov es, dx

no_incr_es:
	pop di
	pop si
	pop dx
	pop ecx
	ret

; ===================================================================================================
; EOF

