;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
; Text functions
;
; Based on long mode examples by Mac2004
; available at FASM board.
;-----------------------------------------------------------------------------------


;----------------------------------------------------------
; RM_PrintString
; Dumps DS:SI to the screen using video BIOS call
;
; input  : si - string to print
; output : -- 
;----------------------------------------------------------

RM_PrintString:

    pusha
    
.DoItAgain:

    lodsb                          ; Load byte at DS:SI into AL
    or al,al                       ; Test if character is 0
                                   ; (end of string)
    jz .Done                       ; Go to Done if Zero

.BIOS:

    mov ah,0Eh                     ; Put character on screen at the
                                   ; current cursor location
    mov bx,7                       ; Video attribute for the character
    int 10h                        ; Call BIOS

    jmp .DoItAgain

.Done:

    popa
    
    ret                            ; Return to caller

;-------------------------------------------------------------------------------
; RM_ToHex
; Loads HexBuffer with ASCII corresponding to 8, 16, or 32 bit interger in hex.
;
; input  : al, ax, or eax - number to convert (depending on bit size)
;          cl - the number of bits  
; output : a full buffer or an empty buffer on error
;-------------------------------------------------------------------------------
HexBuffer               db 18       	    ; Sixteen bytes with the ending zero

RM_ToHex:

    pusha

    mov bx, 0                                ; Load BX with pointer offset
    mov edx, eax                             ; Save the EAX to EDX
    cmp cl, 8                                ; Check for 8 bits
    jne .Check16
    jmp .ConverterLoop                       ; Start loading the buffer

.Check16:

    cmp cl, 10h                              ; Check for 16 bits
    jne .Check32
    jmp .ConverterLoop                       ; Start loading the buffer

.Check32:

    cmp cl, 20h                              ; Check for 32 bits
    jne .ErrorBits

.ConverterLoop:

    mov eax, edx                             ; Reload EAX with the converter
    sub cl, 4                                ; Lower bit count by 4 bits
    shr eax, cl

    and al, 0Fh
    add al, '0'
    cmp al ,'9'
    jbe .LoadBuffer
    add al, 'A'-'0'-10                       ; Convert to "A" to "F"

.LoadBuffer:

    mov [HexBuffer + bx], al                 ; Load buffer with AL
    inc bx                                   ; Increment buffer pointer
    cmp cl, 0                                ; Check if we're done
    jne .ConverterLoop                       ; Do next byte

.ErrorBits:

    mov byte [HexBuffer + bx], 0             ; End the string with a zero
    
    popa

    ret

;-----------------------------------------------------------------------------------
; RM_ToDecimal - converts whatever is in EAX to ASCII string (places it in a buffer)
;
; input  : eax - number to convert
; output : a full buffer
;-----------------------------------------------------------------------------------
ZeroNotLoaded   db 1        ; Used for checking zero starting numbers
DivisorNumber   dw 1        ; Used to keep track of division
DecimalBuffer   db 12       ; Eleven bytes with the ending zero
DivisorTable    dd 1000000000,100000000,10000000,1000000,100000,10000,1000,100,10

RM_ToDecimal:

    pusha

    mov byte [ZeroNotLoaded], 0              ; Initialize variable
    mov word [DivisorNumber], 0              ; Initialize variable
    mov bx, 0                                ; Start BX at zero

.LoopToDecimal:

    mov edx, 0
    push eax
    movzx eax, word [DivisorNumber]
    mov ecx, 4
    mul ecx
    push bx
    mov bx, ax
    mov ecx, dword [DivisorTable + bx]
    pop bx
    pop eax
    div ecx
    inc word [DivisorNumber]
    cmp al,0
    je .CheckZeroNotLoaded
    
.ReturnToLoop:

    add al,48                                ; Update to an ASCII character representation
    mov [DecimalBuffer + bx], al             ; Load decimal buffer
    mov byte [ZeroNotLoaded], 1              ; Gone through the loop at least once, show it
    inc bx                                   ; Increment the BX
    cmp ecx, dword [DivisorTable + (8 * 4)]  ; Was that the last divisor?
    je .MoveOut                              ; Yep, do the one's column. :D
    mov eax, edx                             ; Put remainder in EAX
    jmp .LoopToDecimal
    
.CheckZeroNotLoaded:

    cmp byte [ZeroNotLoaded], 1
    je .ReturnToLoop
    mov eax, edx
    jmp .LoopToDecimal
    
.MoveOut:

    mov eax, edx                             ; Put remainder in EAX
    add al, 48                               ; Update to an ASCII character representation
    mov [DecimalBuffer + bx], al             ; Load decimal buffer
    inc bx                                   ; Increment the BX
    mov dl, 0
    mov [DecimalBuffer + bx], dl             ; Load decimal buffer with ending zero

    popa

    ret


;----------------------------------------------------------
; RM_PutStr  - Prints asciiz string
;
; input:  ax - startaddress in bytes
;         bx - Ptr to str
;         ch - color
; output: --
;----------------------------------------------------------

RM_PutStr:

        push es
	pusha

	;write string one char at the time
	;------------------------------------
.loop:
        mov cl,byte[bx]     	;get next char

        cmp cl,0                ;Last character?
        jz .done

	call RM_PutChar
        inc bx                  ;point to next char
        add ax,1                ;next coordinate
        jmp .loop

.done:
	popa
        pop es
        ret       		; end of procedure


;----------------------------------------------------------
; RM_PutChar - Prints one  character.
;
; input:  ax - startaddress in bytes
;         cx - char to be printed
; output: --
;----------------------------------------------------------

RM_PutChar:

        push es
	pusha

     	mov dx,ax 		;save start address

        push ax
        mov ax,0xB800
        mov es, ax
        pop ax
        mov bx,0

        add ax,dx	

	;write actual char
	;------------------
	
        add bx, ax         	;get start position
        mov [es:bx],cx     	;print char
        shr bx,1           	
.done:
	popa
        pop es
        ret       		; end of procedure



;----------------------------------------------------------
; RM_Wait    - Wait n seconds
;
; input:  bx - num. seconds
; output: --
;----------------------------------------------------------
ticks dw 0

RM_Wait:
	pusha
.start:
	mov ah, 0x00		; CX,DX Ticks since last reset
	int 0x1a		; CX is high, DX is low
	
	mov [ticks], dx
.again:
	mov ah, 0x00
	int 0x1a

	mov cx, [ticks]
	sub dx, cx
	cmp dx, 18
	jb .again

	dec bx
	cmp bx, 0
	jne .start	

	popa
	ret


; The tick count is stored at 0040:006E in the BIOS Data Area.
; It is incremented about once every 55 ms by INT 08H:
; clock updates at 1193180/65536 (about 18.2) ticks per second.
;
;            counts per second  18
;            counts per minute  1092
;            counts per hour    65543
;            counts per day     1573040


;-------------------------------------
; RM_clear_keyboard_buffer
;
;-------------------------------------
RM_clear_keyboard_buffer:

push ax
push es

mov ax, 0000h
mov es, ax

mov word[es:041ah], 0x041e 	; Keyboard buffer start and end
mov word[es:041ch], 0x041e 	; Clears keyboard buffer

pop es
pop ax

ret

; The BIOS has a default memory area for the keyboard buffer which holds up to 16 keystrokes.
; This area starts at 0040:001Eh and is 32 bytes in length. (16 2-byte scancodes)
; The word (2 bytes) at 0040:001Ah hold the current head of the BIOS keyboard buffer.
; The word (2 bytes) at 0040:001Ch hold the current tail of the BIOS keyboard buffer.
; If these two pointers are different, then there is a keystroke ready for extraction.


; ===================================================================================================
; EOF


