;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
;
; Init SMP AP
;-----------------------------------------------------------------------------------

USE16

trampoline:
	nop
	jmp 0x0000:clearcs_ap
	
clearcs_ap:

	sti						; Enable all interrupts

	; Enable the A20 gate
	;--------------------
	call RM_Init_a20


	cli						; Disable the interrupts

	lgdt [cs:GDTR]					; load GDT register

	mov eax, cr0 					; switch to 32-bit protected mode
	or al, 1
	mov cr0, eax

	jmp	CODE_SELECTOR:startap32

align 16


; =============================================================================

USE32

startap32:
	mov eax, DATA_SELECTOR		; load 4 GB data descriptor
	mov ds, ax			; to all data segment registers
	mov es, ax
	mov fs, ax
	mov gs, ax
	mov ss, ax
	xor eax, eax
	xor ebx, ebx
	xor ecx, ecx
	xor edx, edx
	xor esi, esi
	xor edi, edi
	xor ebp, ebp
	mov esp, 0x8000			; Set a known free location for the stack


	; Load the GDT
	lgdt [GDTR]

	; PAE, CR3, LME, CR0
	;-------------------
	call PM_Init_CR

	jmp LONG_SELECTOR:startap64	

align 16


; =============================================================================
; 64-bit mode
USE64

startap64:	

	mov rax, clearcs64_ap
	jmp rax
	nop
clearcs64_ap:
	xor rax, rax

	; Reset the stack. Each CPU gets a 4096-byte unique stack location
	mov rsi, [os_LocalAPICAddress]	; We would call os_smp_get_id here but the stack is not ...
	add rsi, 0x20			; ... yet defined. It is safer to find the value directly.
	lodsd				; Load a 32-bit value. We only want the high 8 bits
	shr rax, 24			; Shift to the right and AL now holds the CPU's APIC ID
	shl rax, 12			; shift left 12 bits for a 4096 byte stack
	add rax, 0x0000000000201000	; stacks decrement when you "push", start at 4096 bytes in
	mov rsp, rax			; (BSP's stack reset in SMP.inc)

	lgdt [GDTR]			; Load the GDT
	lidt [IDTR]			; load IDT register

; Enable Local APIC on AP
	mov rsi, [os_LocalAPICAddress]
	add rsi, 0x00f0			; Offset to Spurious Interrupt Register
	mov rdi, rsi
	lodsd
	or eax, 0000000100000000b
	stosd


	lock
	inc word [cpu_activated]
	xor eax, eax
	mov rsi, [os_LocalAPICAddress]
	add rsi, 0x20			; Add the offset for the APIC ID location
	lodsd				; APIC ID is stored in bits 31:24
	shr rax, 24			; AL now holds the CPU's APIC ID (0 - 255)
	mov rdi, 0x00005700		; The location where the cpu values are stored
	add rdi, rax			; RDI points to infomap CPU area + APIC ID. ex F701 would be APIC ID 1
	mov al, 1
	stosb

	sti				; Activate interrupts for SMP
	jmp ap_sleep


ap_sleep:
	hlt				; Suspend CPU until an interrupt is received. opcode for hlt is 0xF4
	jmp ap_sleep			; just-in-case of an NMI


; =============================================================================
; EOF
