;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
; VESA
;
; Partially based on long mode examples by Mac2004
; available at FASM board.
;-----------------------------------------------------------------------------------


align 4

;---------------------------------------------------
;  {VESA VBE modeInfoBlock}
;---------------------------------------------------

struc VBE_MODEINFOBLOCK_STRUCTURE 
	{

	;general fields
	;---------------

      	.modeattr 	dw 0    	;  0 : Mode attributes
      	.wina  		db 0     	;  2 : Window A attributes
      	.winb  		db 0     	;  3 : Window B attributes
      	.gran  		dw 0     	;  4 : Window granularity in KiB
      	.winsize	dw 0     	;  6 : Window size in KiB
      	.sega  		dw 0     	;  8 : Start segment of window A
      	.segb  		dw 0     	; 10 : Start segment of window B        
      	.winfunc_adr 	dd 0    	; 12 : Pointer FAR (real mode) to window positioning function (equ set bank: AX=4F05h)
      	.bytesperline 	dw 0		; 16 : Bytes per scan line

        ;---remainder is optional for VESA modes, needed for OEM modes---
        .xres 		dw 0   		; 18 : Width in pixels
        .yres	 	dw 0  		; 20 : Height in pixels
        .charw 		db 0   		; 22 : Width of character cell in pixels
        .charh 		db 0   		; 23 : Height of character cell in pixels
        .planes 	db 0   		; 24 : Number of memory planes
        .bpp  		db 0      	; 25 : Number of bits per pixel
        .banks	 	db 0     	; 26 : Number of banks
        .mmodel 	db 0      	; 27 : Memory model type
        .banksize	db 0      	; 28 : Size of bank in KiB
        .images 	db 0     	; 29 : Number of image pages
        .reserved   	db 0        	; 30 : Reserved(1)

        ;------VBE v1.2+ ------ Direct Color fields (required for direct/6 and YUV/7 memory models)
        .redmasksize 	db 0       	; 31 : Red mask - size of direct color (#bits)
        .redfieldpos 	db 0       	; 32 : Red mask - bit position of lsb
        .greenmasksize 	db 0     	; 33 : Green mask - size of direct color (#bits)
        .greenfieldpos 	db 0     	; 34 : Green mask - bit position of lsb
        .bluemasksize 	db 0      	; 35 : Blue mask - size of direct color (#bits)
        .bluefieldpos 	db 0      	; 36 : Blue mask - bit position of lsb
        .rsvdmasksize 	db 0      	; 37 : Reserved mask - size of direct color (#bits)
        .rsvdfieldpos 	db 0      	; 38 : Reserved mask - bit position of lsb
        .dcolorinf   	db 0     	; 39 : DirectColorModeInfo

        ;------VBE v2+ ------------------------------------------------------------------- 
        .LFB	 	dd 0       	; 40 : Physical address of linear video buffer
        .offscreen	dd 0     	; 44 : Offset (pointer) to start of offscreen memory
        .offsize	dw 0     	; 48 : Size of offscreen memory in KiB

	;-- Mandatory information for VBE 3.0 and above-----------------------------------
        .linbytesperline   dw 0      	; 50 : Bytes per scan line (linear modes)
        .bankedpages 	   db 0       	; 52 : Number of pages for banked modes
        .linearpages 	   db 0       	; 53 : Number of pages for linear modes
        .linredmasksize    db 0       	; 54 : Red mask - size of direct color (linear modes)
        .linredfieldpos    db 0       	; 55 : Red mask - bit position of lsb (linear modes)
        .lingreenmasksize  db 0     	; 56 : Green mask - size of direct color (linear modes)
        .lingreenfieldpos  db 0     	; 57 : Green mask - bit position of lsb (linear modes)
        .linbluemasksize   db 0      	; 58 : Blue mask - size of direct color  (linear modes)
        .linbluefieldpos   db 0      	; 59 : Blue mask - bit position of lsb (linear modes)
        .linrsvdmasksize   db 0      	; 60 : Reserved mask - size of direct color (linear modes)
        .linrsvdfieldpos   db 0      	; 61 : Reserved mask - bit position of lsb (linear modes)
        .maxpixelclock 	   dd 0     	; 62 : Maximum pixel clock (in Hz) for graphics mode
        .OEMbuffer: 	times 190 db 0  ; 66 : OEM scratchpad

	};end of structure (256 bytes)

;-------------------------------------------------------------------
;  {VESA VBE INFO_BLOCK}
;-------------------------------------------------------------------

struc VBE_INFO_BLOCK_STRUCTURE 
	{
	;general fields!
	;---------------

        .sign 		dd 0      	;  0 : {Must be 'VESA'}
        .vers 		dw 0      	;  4 : {VBE version number}
        .oem	 	dd 0    	;  6 : pointer to OEM name
        .capab 		dd 0     	; 10 : capabilities
        .modelist 	dd 0  		; 14 : pointer to list of supported VESA and
                       			;      OEM video modes Terminated with 0FFFFh
        .memory  	dw 0   		; 18 : Video memory in 64k blocks

	;Remaining fields are for VBE 2.0 or later
	;------------------------------------------

        .oemver 	dw 0   		; 20 : {OEM version}
        .vendor 	dd 0   		; 22 : {Vendor name}
        .product 	dd 0  		; 26 : {Product Name}
        .prodrev 	dd 0    	; 30 : {Product Revision}
	.reserved:	times 222 db 0  ; 34 : reserved for VBE
	.OEMdata:	times 256 db 0  ;256 : OEM scratchpad

	};end of structure

align 4
         Vesa_info 		VBE_INFO_BLOCK_STRUCTURE        ; Structure for VBE info block.
         Vesa_modeinfo	 	VBE_MODEINFOBLOCK_STRUCTURE	; Structure for obtaining specific mode info.


;------------------------------------------------------------



;-------------------------------------------------------------------
; RM_Check_VBE  -   See if there's a VESA BIOS present
;
; input:  --
; output: --
;-------------------------------------------------------------------

RM_Check_VBE:

	push es
	pusha

	;See if there's a VESA BIOS
	;---------------------------

	;-----------------------------------------------------------
	;Round 1... Vesa_info.sign needs to contain 'VESA'=$32454256
	;after the bios call.
	;-----------------------------------------------------------
	push es
	mov  ax, 0x4f00				; get VBE controller information.
	mov  di, Vesa_info			; use our struc to save the info.
	push cs
	pop  es
	int  10h				; do it!
	pop  es

	cmp  ax, 0x004f				; test result.
	je .VBEFound

	mov ax, 200+7				; start point
	mov ch, ' '				; color
	mov bx, vbe_not_ok_msg			; str
	call RM_PutStr

	jmp .continue
.VBEFound:

	mov ax, 200+7				; start point
	mov ch, ' '				; color
	mov bx, vbe_ok_msg			; str
	call RM_PutStr

.continue:
	; let's save "VBE2" to see, if above vesa2.0
	;----------------------------------------------
	mov eax, 032454256h			; don't remove, will crash totally!!
	mov dword[Vesa_info.sign], eax		; "VBE2" - enable VBE2 if present

	mov ax, 0x4f00				; 4F00h --GET SuperVGA INFORMATION
	mov di, Vesa_info			; get buffer pointer
	int 10h					; do it

	cmp ax, 0x004f				; if ok with the query ?
	jne .vbe2_fail				; not ok with vbe2+!


	mov eax, 41534556h               	; test for vbe2 result.
	cmp dword[Vesa_info.sign], eax		; "VBE2" - enable VBE2 if present
	jne .vbe2_fail

	;Here's everything ok!!!

	mov ax, 200+80+7			; start point
	mov ch, ' '				; color
	mov bx, vbe2_ok_msg			; str
	call RM_PutStr
	jmp .vbe2ok

	;only old vbe1+ card was located ;--> let's tell the user.
	;----------------------------------------------------------

.vbe2_fail:
	mov ax, 200+80+7			; start point
	mov ch, ' '				; color
	mov bx, vbe2_not_ok_msg			; str
	call RM_PutStr

	;Ask info on requested vesa mode
	;-------------------------------
.vbe2ok:

.done:
	popa
	pop es
	ret       				; end of procedure


;-------------------------------------------------------------------
; RM_Init_VBE - Initialize LFB mode
;
; input:  --
; output: --
;-------------------------------------------------------------------
RM_Init_VBE:

	push es
	pusha

	;init selected LFB-mode here!
	;----------------------------

	xor ebx, ebx
	mov bx, [VESAMODE]				; * Default *
	or  ebx, 04000h					; Toggle lfb flag on
	mov ax, 0x4f02					; init vesamode function
	int 10h

	;Ask info on requested vesa mode
	;-------------------------------

	mov cx, [VESAMODE]				; get mode info structure
	mov di, Vesa_modeinfo				; pointer to mode info block
	mov ax, 0x4F01					; Get VBE Mode Information
	int 10h						; Do it!

.done:
	popa
	pop es
	ret       					; end of procedure


;-------------------------------------------------------------------
; RM_Info_VBE - Fill table with modeinfo for all modes and
;		set override modes
; input:  --
; output: --
;-------------------------------------------------------------------
RM_Info_VBE:

	push fs
	push gs
	push es
	pusha
	
	mov ax, 2000h			; table address (segment)
	push ax
	pop gs

	mov eax, dword[Vesa_info.modelist]
	mov bx, ax			; offset
	shr eax, 16
	mov fs, ax			; segment (just in case, seg=0)

	mov di, 0			; 0=0x20000 start segment

	; copy the first 66 bytes of modeinfo x all modes
.nextlist:
	xor eax, eax
	mov ax, word[fs:bx]		; [VESAMODE]
	cmp ax, 0xFFFF
	je .endlist

	call RM_Ask_VBE			; fill Vesa_modeinfo
	call RM_Fover_VBE		; set F1-F10 override modes

	mov si, Vesa_modeinfo
	mov cx, 33 			; 66/2
.copyt2:
	mov dx, word[si] 
	mov word[gs:di], dx
	add si, 2
	add di, 2
	loop .copyt2

	mov word[gs:di], ax		; [VESAMODE]
	add di, 2			; bytes n.67-68

	add bx, 2
	jmp .nextlist

.endlist:

.done:
	popa
	pop es
	pop gs
	pop fs
	ret       			; end of procedure


;-------------------------------------------------------------------
; RM_Ask_VBE - Get VBE Mode Information
; IN: ax = [VESAMODE]
;-------------------------------------------------------------------
RM_Ask_VBE:
	push fs
	push gs
	push es
	pusha

	;Ask info on vesa mode
	;------------------------
	mov cx, ax			; get mode info structure
	mov di, Vesa_modeinfo		; pointer to mode info block
	mov ax, 0x4F01			; Get VBE Mode Information
	int 10h				; Do it!

	popa
	pop es
	pop gs
	pop fs
	ret       			; end of procedure

;-------------------------------------------------------------------
; RM_Fover_VBE - Search and set vesamodes
; IN: ax = vesamode
;-------------------------------------------------------------------
RM_Fover_VBE:
	pusha

.F1:					; 1280x720x32
	mov cx, 1280
	mov dx, 720
	call RM_Ftest_VBE
	jc .F2
	mov word[F1mode], ax		; vesamode
	jmp .end


.F2:					; 1600x900x32
	mov cx, 1600
	mov dx, 900
	call RM_Ftest_VBE
	jc .F3
	mov word[F2mode], ax		; vesamode
	jmp .end


.F3:					; 1920x1080x32
	mov cx, 1920
	mov dx, 1080
	call RM_Ftest_VBE
	jc .F4
	mov word[F3mode], ax		; vesamode
	jmp .end


.F4:					; 2560x1440x32
	mov cx, 2560
	mov dx, 1440
	call RM_Ftest_VBE
	jc .F5
	mov word[F4mode], ax		; vesamode
	jmp .end


.F5:					; 1280x800x32
	mov cx, 1280
	mov dx, 800
	call RM_Ftest_VBE
	jc .F6
	mov word[F5mode], ax		; vesamode
	jmp .end


.F6:					; 1680x1050x32
	mov cx, 1680
	mov dx, 1050
	call RM_Ftest_VBE
	jc .F7
	mov word[F6mode], ax		; vesamode
	jmp .end


.F7:					; 1920x1200x32
	mov cx, 1920
	mov dx, 1200
	call RM_Ftest_VBE
	jc .F8
	mov word[F7mode], ax		; vesamode
	jmp .end


.F8:					; 2560x1600x32
	mov cx, 2560
	mov dx, 1600
	call RM_Ftest_VBE
	jc .F9
	mov word[F8mode], ax		; vesamode
	jmp .end


.F9:					; 800x600x32
	mov cx, 800
	mov dx, 600
	call RM_Ftest_VBE
	jc .F10
	mov word[F9mode], ax		; vesamode
	jmp .end


.F10:					; 1024x768x32
	mov cx, 1024
	mov dx, 768
	call RM_Ftest_VBE
	jc .end
	mov word[F10mode], ax		; vesamode
	jmp .end


.end:
	popa
	ret       			; end of procedure


;-------------------------------------------------------------------
; RM_Ftest_VBE - Test vesamode available
; IN:	cx = xres
;	dx = yres
; OUT: 	set carry (0=OK, 1=not)
;-------------------------------------------------------------------
RM_Ftest_VBE:

	cmp word[Vesa_modeinfo.xres], cx
	jne .not
	cmp word[Vesa_modeinfo.yres], dx
	jne .not
	cmp byte[Vesa_modeinfo.mmodel], 6	; direct color
	jne .not
	cmp byte[Vesa_modeinfo.bpp], 32		; 32bits
	jne .not
	bt word[Vesa_modeinfo.modeattr], 0	; available
	jnc .not
	bt word[Vesa_modeinfo.modeattr], 1	; 1.2+
	jnc .not
	bt word[Vesa_modeinfo.modeattr], 3	; color
	jnc .not
	bt word[Vesa_modeinfo.modeattr], 4	; graphic
	jnc .not
	bt word[Vesa_modeinfo.modeattr], 7	; LFB
	jnc .not

	clc
	jmp .end
.not:
	stc
.end:
	ret


;-------------------------------------------------------------------
; RM_Override_VBE - Set the selected vesamode at runtime
; IN: ah = scancode key pressed
;-------------------------------------------------------------------
RM_Override_VBE:
	push ax
	push bx

	mov [GUI], 1			; GUI on

	movzx bx, ah
	sub bx, 59			; use ah as index (59->68)
	shl bx, 1			; mul 2 (word)
	mov ax, word[FNmodes+bx]
	cmp ax, 0
	je .end
	mov [VESAMODE], ax		; selected mode

	shr bx, 1
	inc bx
	mov [IDMODE], bl		; ID of selected mode

	;jmp .end
.end:
	pop bx
	pop ax
	ret



; ===================================================================================================
; EOF

; VBE Return Status Info
; OUT: al
; VBE_RETURN_STATUS_SUPPORTED                      0x4F
; VBE_RETURN_STATUS_UNSUPPORTED                    0x00 (not 0x4F)
; OUT: ah
; VBE_RETURN_STATUS_SUCCESSFULL                    0x00
; VBE_RETURN_STATUS_FAILED                         0x01
; VBE_RETURN_STATUS_NOT_SUPPORTED                  0x02
; VBE_RETURN_STATUS_INVALID                        0x03

;-------------------------------------------------------------------------
; From VBE20.txt Core Functions Standard
;
; 4.5. Function 02h  - Set VBE Mode
;
;     This required function initializes the controller and sets a VBE mode.
;     The format of VESA VBE mode numbers is described earlier in this
;     document. If the mode cannot be set, the BIOS should leave the
;     graphics environment unchanged and return a failure error code.
;
;     Input:    AX   = 4F02h     Set VBE Mode
;               BX   =           Desired Mode to set
;                    D0-D8  =    Mode number
;                    D9-D13 =    Reserved (must be 0)
;                    D14    = 0  Use windowed frame buffer model
;                           = 1  Use linear/flat frame buffer model
;                    D15    = 0  Clear display memory
;                           = 1  Don't clear display memory
;
;     Output:   AX   =           VBE Return Status
;
;     Note: All other registers are preserved.
;
;     If the requested mode number is not available, then the call will
;     fail, returning AH=01h to indicate the failure to the application.
;
;     If bit D14 is set, the mode will be initialized for use with a flat
;     frame buffer model.  The base address of the frame buffer can be
;     determined from the extended mode information returned by VBE Function
;     01h.  If D14 is set, and a linear frame buffer model is not available
;     then the call will fail, returning AH=01h to the application.
;
;     If bit D15 is not set, all reported image pages, based on Function 00h
;     returned information NumberOfImagePages, will be cleared to 00h in
;     graphics mode, and 20 07 in text mode.  Memory over and above the
;     reported image pages will not be changed.  If bit D15 is set, then the
;     contents of the frame buffer after the mode change is undefined.
;     Note, the 1-byte mode numbers used in Function 00h of an IBM VGA
;     compatible BIOS use D7 to signify the same thing as D15 does in this
;     function.  If D7 is set for an IBM compatible mode set using this
;     Function (02), this mode set will fail.  VBE aware applications must
;     use the memory clear bit in D15.
;
;     Note: VBE BIOS 2.0 implementations should also update the BIOS Data
;     Area 40:87 memory clear bit so that VBE Function 03h can return this
;     flag. VBE BIOS 1.2 and earlier implementations ignore the memory clear
;     bit.
;
;     Note:  This call should not set modes not listed in the list of
;     supported modes.  In addition all modes (including IBM standard VGA
;     modes), if listed as supported, must have ModeInfoBlock structures
;     associated with them.  Required ModeInfoBlock values for the IBM
;     Standard Modes are listed in Appendix 2.
;
;     Note: If there is a failure when selecting an unsupported D14 value,
;     the error return should be 02h.

;--------------------------------------------
; Note: The VESA numbers are NOT standard!
;
; VBE GFX Mode Number
;				Hex	Dec
;--------------------------------------------
; VESA_640x400x8		0x100	256
; VESA_640x480x8                0x101	257
; VESA_800x600x4                0x102	258
; VESA_800x600x8                0x103	259
; VESA_1024x768x4               0x104	260
; VESA_1024x768x8               0x105	261
; VESA_1280x1024x4              0x106	262
; VESA_1280x1024x8              0x107	263
; VESA_320x200x1555             0x10D	269
; VESA_320x200x565              0x10E	270
; VESA_320x200x888              0x10F	271
; VESA_640x480x1555             0x110	272
; VESA_640x480x565              0x111	273
; VESA_640x480x888              0x112	274
; VESA_800x600x1555             0x113	275
; VESA_800x600x565              0x114	276
; VESA_800x600x888              0x115	277
; VESA_1024x768x1555            0x116	278
; VESA_1024x768x565             0x117	279
; VESA_1024x768x888             0x118	280
; VESA_1280x1024x1555           0x119	281
; VESA_1280x1024x565            0x11A	282
; VESA_1280x1024x888            0x11B	283
;
; 1600x1200x8              	0x11C	284
; 1600x1200x1555           	0x11D	285
; 1600x1200x565            	0x11E	286
; 1600x1200x888            	0x11F	288
;
; Other used numbers
; 1920x1440x8			0x13C	316
; 1920x1440x565			0x14D	333
; 1920x1440x8888		0x15C	348
; 1600x1200x8			0x13A	314
; 1600x1200x565			0x14B	331
; 1600x1200x8888		0x15A	346

; QEMU-KVM
; QEMU_640x480x8888		0x142	322	
; QEMU_800x600x8888		0x143	323
; QEMU_1024x768x8888            0x144	324
; QEMU_1280x1024x8888           0x145	325
; QEMU_1600x1200x8888           0x147	327
; QEMU_1152x864x8888            0x14c	332

; BOCHS/PLEx86 'own' mode numbers
; BOCHS_320x200x8888            0x140	320
; BOCHS_640x400x8888            0x141	321
; BOCHS_640x480x8888            0x142	322
; BOCHS_800x600x8888            0x143	323
; BOCHS_1024x768x8888           0x144	324
; BOCHS_1280x1024x8888          0x145	325
; BOCHS_320x200x8               0x146	326
; BOCHS_1600x1200x8888          0x147	327
; BOCHS_1152x864x8              0x148	328
; BOCHS_1152x864x1555           0x149	329
; BOCHS_1152x864x565            0x14a	330
; BOCHS_1152x864x888            0x14b	331
; BOCHS_1152x864x8888           0x14c	332

