;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
; Paging
;-----------------------------------------------------------------------------------

PML4 equ 0x00002000		; 4096 = 4KiB
PDP  equ PML4 + 0x00001000 	; 4096 = 4KiB
PD   equ 0x00100000

PM_Init_Paging:

; Clear out 2*4096 bytes of memory. This will store the 64-bit PML4, and PDP
	xor eax, eax
	mov edi, PML4
	mov ecx, 2*1024
	rep stosd

; Clear memory for the Page Descriptor Entries (0x00100000 - 0x0013FFFF)
	xor eax, eax
	mov edi, PD
	mov ecx, 65536			; 256 KiB
	rep stosd


; Create the Level 4 Page Map. (Maps 64GBs of 2MB pages)
; First create a PML4 entry.
; PML4 is stored at 0x0000000000002000, create the first entry there
; A single PML4 entry can map 512GB with 2MB pages.
	cld
	mov edi, PML4			; Create a PML4 entry for the first 64GB of RAM
	mov eax, PDP + 00000111b
	stosd
	xor eax, eax
	stosd

	; mov edi, 0x00002800		; Create a PML4 entry for higher half (starting at 0xFFFF800000000000)
	; mov eax, 0x00003007		; The higher half is identity mapped to the lower half
	; stosd
	; xor eax, eax
	; stosd

; Create the PDP entries.
; The first PDP is stored at 0x0000000000003000, create the first entries there
; A single PDP entry can map 1GB with 2MB pages
	mov ecx, 64			; number of PDPE's to make.. each PDPE maps 1GB of physical memory
	mov edi, PDP
	mov eax, PD + 00000111b		; location of first PD
create_pdpe:
	stosd
	push eax
	xor eax, eax
	stosd
	pop eax
	add eax, 0x00001000		; 4K later (512 records x 8 bytes)
	dec ecx
	cmp ecx, 0
	jne create_pdpe

; Create the PD entries.
; PD entries are stored starting at 0x0000000000100000 and ending at 0x000000000013FFFF (256 KiB)
; This gives us room to map 64 GiB with 2 MiB pages
	mov edi, PD
	mov eax, 0x00000000 + 10001111b	; Bit 7 must be set to 1 as we have 2 MiB pages
	xor ecx, ecx
pd_again:				; Create a 2 MiB page
	stosd
	push eax
	xor eax, eax
	stosd
	pop eax
	add eax, 0x00200000
	inc ecx
	cmp ecx, 2048
	jne pd_again			; Create 2048 2 MiB page maps. (4 GiB)


ret


; ===================================================================================================
; EOF

;------------------------------------------------------------------------------
;
;
; PML4T
;             
; one table  |   1   |   2   | .............................. |  512  |  = 4 KiB
;             8bytes  8bytes           ................        8bytes
;            |   |
;            |   +-> 512 GiB addressing                           +-> Max 256 TiB addressable
;            |
;            |
; PDPT       V
;             
; one table  |   1   |   2   | .............................. |  512  |  = 4 KiB
;             8bytes  8bytes           ................        8bytes
;            |   |
;            |   +-> 1 GiB addressing                             +-> Max 512 GiB addressable
;            |
;            |
; PDT        V
;             
; one table  |   1   |   2   | .............................. |  512  |  = 4 KiB
;             8bytes  8bytes           ................        8bytes
;            |   |
;            |   +-> 2 MiB addressing                             +-> Max 1 GiB addressable
;            |
;            |
; PT         V
;             
; one table  |   1   |   2   | .............................. |  512  |  = 4 KiB
;             8bytes  8bytes           ................        8bytes
;            |   |
;            |   +-> 4 KiB addressing                             +-> Max 2 MiB addressable
;            |
;            |
;            +--> Physical Address
;
;
;---------------------------------------------------------------------------------




;
; 2 MiB PML4E Long Mode
;
;  63 62 61 60 59 58 57 56 55 54 53 52 51 50 49 48 47 46 45 44 43 42 41 40 39 38 37 36 35 34 33 32
;  -----------------------------------------------------------------------------------------------
; |N |                                |                                                           |
; |X |         Available              |      Page Dyrectory Pointer Base Address                  | +0
; |  |                                |                                                           |
;  -----------------------------------------------------------------------------------------------
; |                                                           |        |     |  |  |P |P |U |R |  |
; |      Page Dyrectory Pointer Base Address                  | AVL    |     |  |A |C |W |/ |/ |P | +4
; |                                                           |        |     |  |  |D |T |S |W |  |
;  -----------------------------------------------------------------------------------------------
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;
;
;
; 2 MiB PDPE Long Mode
;
;  63 62 61 60 59 58 57 56 55 54 53 52 51 50 49 48 47 46 45 44 43 42 41 40 39 38 37 36 35 34 33 32
;  -----------------------------------------------------------------------------------------------
; |N |                                |                                                           |
; |X |         Available              |      Page Dyrectory Pointer Base Address                  | +0
; |  |                                |                                                           |
;  -----------------------------------------------------------------------------------------------
; |                                                           |        |  |  |  |  |P |P |U |R |  |
; |      Page Dyrectory Pointer Base Address                  | AVL    |  |0 |  |A |C |W |/ |/ |P | +4
; |                                                           |        |  |  |  |  |D |T |S |W |  |
;  -----------------------------------------------------------------------------------------------
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;
;
;
; 2 MiB PDE Long Mode
;
;  63 62 61 60 59 58 57 56 55 54 53 52 51 50 49 48 47 46 45 44 43 42 41 40 39 38 37 36 35 34 33 32
;  -----------------------------------------------------------------------------------------------
; |N |                                |                                                           |
; |X |         Available              |      Page Dyrectory Pointer Base Address                  | +0
; |  |                                |                                                           |
;  -----------------------------------------------------------------------------------------------
; |                                |                       |P |        |  |  |  |  |P |P |U |R |  |
; |   Physical Page Base address   |                       |A | AVL    |G |1 |D |A |C |W |/ |/ |P | +4
; |                                |                       |T |        |  |  |  |  |D |T |S |W |  |
;  -----------------------------------------------------------------------------------------------
;  31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00
;
;
; Present (P) Bit. Bit 0.
; P bit = cleared to 0 => the page-translation table or physical page is not loaded in physical memory.
; P bit = set to 1  => the page-translation table or physical page is loaded in physical memory.
;
; Read/Write (R/W) Bit. Bit 1. 
; read/write access to all physical pages mapped by the table entry, through the levels.
; R/W bit = cleared to 0 => access is restricted to read-only.
; R/W bit = set to 1 => both read and write access is allowed.
;
; User/Supervisor (U/S) Bit. Bit 2.
; U/S bit = set to 1 => access to all physical pages mapped is restricted to user level (CPL 3). 
; U/S bit = cleared to 0 => access to all physical pages mapped is restricted to supervisor level (CPL 0, 1, 2).
;
; Page-Level Writethrough (PWT) Bit. Bit 3.
; PWT bit = cleared to 0 => the table or physical page has a writeback caching policy. 
; PWT bit = set to 1 => the table or physical page has a writethrough caching policy.
;
; Page-Level Cache Disable (PCD) Bit. Bit 4.
; PCD bit = cleared to 0 => the page-translation table or physical page is cacheable.
; PCD bit = set to 1 => the page-translation table or physical page is not cacheable.
;
; Accessed (A) Bit. Bit 5.
; A bit = set to 1 by the processor the first time the page-translation table or physical page
; is either read from or written to.
; A bit is never cleared by the processor.
;
; Dirty (D) Bit. Bit 6. (only in the lowest level of the page-translation hierarchy)
; D bit = set to 1 by the processor the first time there is a write to the physical page.
; D bit is never cleared by the processor.
;
; Page Size (PS) Bit. Bit 7.
; If PDPE.PS=1 or PDE.PS=1 that entry is the lowest level of the page-translation hierarchy.
; If PS bit cleared to 0 in all levels, the lowest level is PTE => the physical-page size is 4 Kbytes.
; If EFER.LMA=1 and PDPE.PS=1 => the physical-page size is 1 Gbyte.
; If CR4.PAE=0 and PDE.PSE=1  => the physical-page size is 4 Mbytes.
; If CR4.PAE=1 and PDE.PSE=1  => the physical-page size is 2 Mbytes.
; (PDPE)=page-directory-pointer entry, (PDE)=page-directory entry
; 
; Global Page (G) Bit. Bit 8. (only in the lowest level of the page-translation hierarchy)
; G bit = set to 1 => the physical page is a global page.
;
; Available to Software (AVL) Bit.
; Available for system software. Not interpreted by processor.
;
; Page-Attribute Table (PAT) Bit. (only present in the lowest level of the page-translation hierarchy)
; If lowest level = PTE (PDE.PS=0) => PAT occupies bit 7.
; If lowest level = PDE (PDE.PS=1) or PDPE (PDPE.PS=1) => PAT occupies bit 12.
; The PAT bit is the high-order bit of a 3-bit index into the PAT register.
;
; No Execute (NX) Bit. Bit 63.
; In PAE paging (with the exception that the legacy-mode PDPE does not contain this bit). 
; It is not supported by non-PAE paging.
; If EFER.NXE=1 => the NX bit can be set (no-execute page-protection feature enabled)
; If EFER.NXE=0 => the NX bit is treated as reserved.
; If EFER.NXE=0 => a page-fault exception occurs if the NX bit is not cleared to 0.
; This bit controls the ability to execute code.
;
; Reserved Bits. Software should clear all reserved bits to 0.    (Blank)


;------------------------------
; (from Wikipedia)
;
; Virtual address space details
;
; Canonical form addresses
;
; Although virtual addresses are 64 bits wide in 64-bit mode, current implementations (and all chips
; known to be in the planning stages) do not allow the entire virtual address space of 2^64 bytes (16 EB)
; to be used.
; AMD therefore decided that, in the first implementations of the architecture, only the least significant
; 48 bits of a virtual address would actually be used in address translation.
; Further, bits 48 through 63 of any virtual address must be copies of bit 47 (in a manner akin to
; sign extension), or the processor will raise an exception.
; Addresses complying with this rule are referred to as "canonical form."
; Canonical form addresses run from 0 through 00007FFF`FFFFFFFF, and
; from FFFF8000`00000000 through FFFFFFFF`FFFFFFFF, for a total of 256 TB of usable virtual address space.
; This "quirk" allows an important feature for later scalability to true 64-bit addressing:
; many operating systems (including, but not limited to, the Windows NT family) take the higher-addressed
; half of the address space (named kernel space) for themselves and leave the lower-addressed half
; (user space) for application code, user mode stacks, heaps, and other data regions.
; The "canonical address" design ensures that every AMD64 compliant implementation has, in effect,
; two memory halves: the lower half starts at 00000000`00000000 and "grows upwards" as more virtual address
; bits become available, while the higher half is "docked" to the top of the address space and
; grows downwards.
;
;------------------------------
; Current 48-bit implementation
;
;
;  FFFFFFF    FFFFFFFF
; +---------------------+
; |                     |
; |Canonical Higher Half|
; |                     |
; ----------------------+
;  FFFF8000   00000000
;
;
;
;
;
;     Noncanonical
;      addresses
;
;
;
;
;
;  00007FFF    FFFFFFFF         ---> 128 TiB
; +---------------------+
; |                     |
; |Canonical Lower Half |
; |                     |
; ----------------------+
;  00000000   00000000
;
;-------------------------------

;-------------------------------
; 56-bit implementation
;
;
;  FFFFFFF    FFFFFFFF
; +---------------------+
; |                     |
; |Canonical Higher Half|
; |                     |
; |                     |
; ----------------------+
;  FF800000   00000000
;
;
;
;     Noncanonical
;      addresses
;
;
;
;  007FFFFF    FFFFFFFF
; +---------------------+
; |                     |
; |                     |
; |Canonical Lower Half |
; |                     |
; ----------------------+
;  00000000   00000000
;
;------------------------------

;-------------------------------
; Full 64-bit implementation
;
;
;  FFFFFFF    FFFFFFFF
; +---------------------+
; |                     |
; |   Higher Half       |
; |                     |
; |                     |
; |                     |
; |                     |
; |                     |
; +---------------------+
; |                     |
; |                     |
; |                     |
; |                     |
; |                     |
; |   Lower Half        |
; |                     |
; ----------------------+
;  00000000   00000000
;
;------------------------------








