;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
;
; Text functions
;-----------------------------------------------------------------------------------


align 8
hex_conversion_table db '0123456789abcdef'
align 8
Hex_number: 	db '0000000000000000',0


;-------------------------------------------------------------------
; LM_WriteTextString   Writes attributed text string.
;  IN :   AX = display x-coordinate of character.
;	  BX = display y-coordinate of character.
;         CH = color attribute.
;	  DX = current screen width in bytes.
;	  RSI = source string to be written,(ends with char 0h(hex)).
;
;  OUT: --
;-------------------------------------------------------------------


LM_WriteTextString:

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	;Start writing characters until the end sign
	;is reached.
        ;------------------------------------------

.loop:
	mov cl, byte[rsi]			;get char from string.
        cmp cl, 0h                              ;test end sign
 	jne .continue	                        ;end sign not yet--> print the character!
 	jmp .done		                ;all letters are done

.continue:
  	                                        ;ax--> x-coordinate, bx--> y-coordinate
                                                ;cx--> character and color value
                                                ;dx--> current screen width in chars
        call LM_WriteChar                       ;all parameters are in place ;)

	inc ax 	                                ;next x-coordinate
	inc rsi					;next char
        jmp  .loop                              ;fetch next character to be printed.

.done:

	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
        ret



;-------------------------------------------------------------------
;LM_WriteChar- This procedure prints one character to text display.
; IN:     AX = x-coordinate
;	  BX = y-coordinate
;         CX = char to be printed
;	  DX = current screen width
; OUT: --
;-------------------------------------------------------------------

LM_WriteChar:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

        mov di,ax                           ;save x-coordinate
        mov si,bx                           ;save y-coordinate

        mov rbx,0xB8000                     ;point text video memory start

        ;Count starting address (=y*screen width+2*x)
        ;---------------------------------------------

        xor eax,eax
        mov  ax, dx                         ;current text width*2
        add  ax, dx                         ;
        mul  si                             ;width*y

        push cx
        mov  cx,ax                          ;save calculated y-position
        add  cx,di  	                    ;get x-coord and add it to y-coordinate
        add  cx,di 	                    ;add x-coord (again 1 char= 2 bytes)
        mov  ax,cx                          ;get start position
        pop  cx
        cwd                                 ;convert ax to eax and add it

.waitLoop:
        add ebx, eax         		    ;get start position
        mov word[es:ebx],cx   		    ;print char

.exitKey:
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
        ret       ; end of procedure


;----------------------------------------------------------------------------
; LM_Write_TXT_HEX32   This procedure writes 32bit hex number to text screen.
;
; IN:    AX = x-coordinate
;	 BX = y-coordinate
;	 CH = color
;	 EDX = 32 bit number
; OUT:   ---
;-----------------------------------------------------------------------------

LM_Write_TXT_HEX32: 

	;----------------------------------
	; Convert number into ascii string.
	;----------------------------------
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	mov rcx, 8      			; 8 hex digits	
	mov rdi,Hex_number			;get ascii number buffer

	mov dword[rdi],'0000'			;clear ascii buffer
	mov dword[rdi+4],'0000'
	mov dword[rdi+8],'0000'
	mov dword[rdi+12],'0000'

.convert_loop:
	mov rsi,hex_conversion_table		;get hex conversion table
	rol edx, 4 				;rotate so that lowest 4 bits are used
	xor rax,rax				;clear previous stuff
	mov al, 0xF				;al = mask 
	and al, dl
	add rsi,rax				;find correct character from the hex table
	mov al,byte[rsi]

	mov 	byte[rdi],al			;fill ascii buffer
	inc 	rdi
	loop  .convert_loop

	;--------------------
	; Print ascii string
	;--------------------

	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	mov rsi, Hex_number
	mov byte[rsi+8],0			;make end signal

        mov dx,80                    		;dx--> current screen width in chars
	push rsi
	call LM_WriteTextString
	pop rsi
	mov byte[rsi+8],'0'			;clear end signal

.done:
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

        ret       		; end of procedure


;---------------------------------------------------------------------------
; LM_Write_TXT_HEX64  This procedure writes 64bit hex number to text screen.
;
; IN:    AX = x-coordinate
;	 BX = y-coordinate
;	 CH = color
;	 RDX = 64 bit number.
; OUT:   ---
;----------------------------------------------------------------------------

LM_Write_TXT_HEX64: 

	;----------------------------------
	; Convert number into ascii string.
	;----------------------------------
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	mov rcx, 16      			;16 hex digits	
	mov rdi,Hex_number			;get ascii number buffer

	mov dword[rdi],'0000'			;clear ascii buffer
	mov dword[rdi+4],'0000'
	mov dword[rdi+8],'0000'
	mov dword[rdi+12],'0000'

.convert_loop:
	mov rsi,hex_conversion_table		;get hex conversion table
	rol rdx, 4 				;rotate so that lowest 4 bits are used
	xor rax,rax				;clear previous stuff
	mov al, 0xF				;al = mask 
	and al, dl
	add rsi,rax				;find correct character from the hex table
	mov al,byte[rsi]

	mov 	byte[rdi],al			;fill ascii buffer
	inc 	rdi
	loop  .convert_loop

	;--------------------
	; Print ascii string
	;--------------------

	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	mov rsi, Hex_number
        mov dx,80                    		;dx--> current screen width in chars
	push rsi
	call LM_WriteTextString
	pop rsi

.done:
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

        ret       		; end of procedure


; ===================================================================================================
; EOF

