;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
; Structures used by File System
;-----------------------------------------------------------------------------------

align 16

LBA_SYSTEM_END   EQU 16423		; From 0 to 16423 the sectors are reserved
LBA_HEADER1 	 EQU LBA_SYSTEM_END + 1
LBA_ROOT_P	 EQU LBA_HEADER1 + 8
LBA_ROOT	 EQU LBA_HEADER1 + 16
LBA_HEADER2 	 EQU LBA_HEADER1 + 24

LBA_ARCHIVES_P	 EQU LBA_HEADER1 + 32
LBA_DATA_P	 EQU LBA_HEADER1 + 40
LBA_DOCUMENTS_P	 EQU LBA_HEADER1 + 48
LBA_IMAGES_P	 EQU LBA_HEADER1 + 56
LBA_MULTIMEDIA_P EQU LBA_HEADER1 + 64
LBA_OTHER_P	 EQU LBA_HEADER1 + 72
LBA_PROGRAMS_P	 EQU LBA_HEADER1 + 80
LBA_SOURCES_P	 EQU LBA_HEADER1 + 88

START_DIRS	 EQU LBA_HEADER1 + 96

LBA_ARCHIVES 	 EQU START_DIRS
LBA_DATA	 EQU START_DIRS + 4064	; 508*8
LBA_DOCUMENTS	 EQU START_DIRS + (4064*2)
LBA_IMAGES	 EQU START_DIRS + (4064*3)
LBA_MULTIMEDIA	 EQU START_DIRS + (4064*4)
LBA_OTHER	 EQU START_DIRS + (4064*5)
LBA_PROGRAMS	 EQU START_DIRS + (4064*6)
LBA_SOURCES	 EQU START_DIRS + (4064*7)

LBA_INIT_FFCC	 EQU START_DIRS + (4064*8)
LBA_INIT_FREE	 EQU LBA_INIT_FFCC + 8

POINTER_SIZE	EQU 8			; The size of one pointer
CC_PTRS_SIZE	EQU 16			; The size of two pointers in the clusters chain (16 bytes = 128 bit)
NUM_POINTERS	EQU 508			; Number of pointers in one cluster
RECORD_SIZE	EQU 128			; The size of each record
CLUSTER_SIZE	EQU 4096		; 8*512
SECTORS_CLUSTER EQU 8
MAX_NUM_RECS	EQU 16255		; Max number of dirs or files


;headerfs_struc:
;		db 'BNFS0001'		; off:0   Signature
;		dq	0		; off:8   File System Size (in cluster)
;		dq	0		; off:16  Free Space Left (in cluster)
;		dq	0		; off:24  Total number of files
;		dq	0		; off:32  Total number of directories
;		dq	0		; off:40  Cluster of Pointers to Root, absolute address (lba)
;		dq	0		; off:48  First free cluster chain abs address
;		dq	0		; off:56  Last free cluster chain abs address
;		dq	0		; off:64  First use counter cluster chain
;		dq	0		; off:72  First free cluster for first use
;
;headerfs_struc_size  dq 80
					; until 512 bytes: not used
					; from second sector (offset:512): journal, not yet available


root_struc:
		dq	8		; off:0   num. directories = num. records-1
		dq	RECORD_SIZE	; off:8   fixed lenght
		dq	0		; off:16  not used
		dq	0		; off:24  not used
		dq	0		; off:32  level 0
		dq	0		; off:40  
		dq	0		; off:48  
		dq	0		; off:56
		dq	0		; off:64
		dq	0		; off:72
		dq	0		; off:80
		dq	0		; off:88
		dq	0		; off:96
		dq	0		; off:104
		dq	0		; off:112
		db 'END0-DIR'		; off:120		- end record 0
		db 'ARCHIVES'
		times 64 db 0										; off:0   name(70), end string(1), tag(1)
		dq	RECORD_SIZE									; off:72  size (bytes)
		dq	0										; off:80  datetime
		dq	LBA_ARCHIVES_P									; off:88  pointer to (first) cluster of pointers, lba [level 1]
		dq	LBA_ARCHIVES_P									; off:96  pointer to (last) cluster of pointers, lba [level 1]
		dq	0										; off:104 not used
		dq	0										; off:112 not used
		dq	0										; off:120 not used
		db 'DATA'
		times 68 db 0										; off:0   name(70), end string(1), tag(1)
		dq	RECORD_SIZE									; off:72  size (bytes)
		dq	0										; off:80  datetime
		dq	LBA_DATA_P									; off:88  pointer to (first) cluster of pointers, lba [level 1]
		dq	LBA_DATA_P									; off:96  pointer to (last) cluster of pointers, lba [level 1]
		dq	0										; off:104 not used
		dq	0										; off:112 not used
		dq	0										; off:120 not used
		db 'DOCUMENTS'
		times 63 db 0										; off:0   name(70), end string(1), tag(1)
		dq	RECORD_SIZE									; off:72  size (bytes)
		dq	0										; off:80  datetime
		dq	LBA_DOCUMENTS_P									; off:88  pointer to (first) cluster of pointers, lba [level 1]
		dq	LBA_DOCUMENTS_P									; off:96  pointer to (last) cluster of pointers, lba [level 1]
		dq	0										; off:104 not used
		dq	0										; off:112 not used
		dq	0										; off:120 not used
		db 'IMAGES'
		times 66 db 0										; off:0   name(70), end string(1), tag(1)
		dq	RECORD_SIZE									; off:72  size (bytes)
		dq	0										; off:80  datetime
		dq	LBA_IMAGES_P									; off:88  pointer to (first) cluster of pointers, lba [level 1]
		dq	LBA_IMAGES_P									; off:96  pointer to (last) cluster of pointers, lba [level 1]
		dq	0										; off:104 not used
		dq	0										; off:112 not used
		dq	0										; off:120 not used
		db 'MULTIMEDIA'
		times 62 db 0										; off:0   name(70), end string(1), tag(1)
		dq	RECORD_SIZE									; off:72  size (bytes)
		dq	0										; off:80  datetime
		dq	LBA_MULTIMEDIA_P								; off:88  pointer to (first) cluster of pointers, lba [level 1]
		dq	LBA_MULTIMEDIA_P								; off:96  pointer to (last) cluster of pointers, lba [level 1]
		dq	0										; off:104 not used
		dq	0										; off:112 not used
		dq	0										; off:120 not used
		db 'OTHER'
		times 67 db 0										; off:0   name(70), end string(1), tag(1)
		dq	RECORD_SIZE									; off:72  size (bytes)
		dq	0										; off:80  datetime
		dq	LBA_OTHER_P									; off:88  pointer to (first) cluster of pointers, lba [level 1]
		dq	LBA_OTHER_P									; off:96  pointer to (last) cluster of pointers, lba [level 1]
		dq	0										; off:104 not used
		dq	0										; off:112 not used
		dq	0										; off:120 not used
		db 'PROGRAMS'
		times 64 db 0										; off:0   name(70), end string(1), tag(1)
		dq	RECORD_SIZE									; off:72  size (bytes)
		dq	0										; off:80  datetime
		dq	LBA_PROGRAMS_P									; off:88  pointer to (first) cluster of pointers, lba [level 1]
		dq	LBA_PROGRAMS_P									; off:96  pointer to (last) cluster of pointers, lba [level 1]
		dq	0										; off:104 not used
		dq	0										; off:112 not used
		dq	0										; off:120 not used
		db 'SOURCES'
		times 65 db 0										; off:0   name(70), end string(1), tag(1)
		dq	RECORD_SIZE									; off:72  size (bytes)
		dq	0										; off:80  datetime
		dq	LBA_SOURCES_P									; off:88  pointer to (first) cluster of pointers, lba [level 1]
		dq	LBA_SOURCES_P									; off:96  pointer to (last) cluster of pointers, lba [level 1]
		dq	0										; off:104 not used
		dq	0										; off:112 not used
		dq	0										; off:120 not used

root_struc_size: dq 	9*RECORD_SIZE		; 1152 bytes

;-----------------------------------------------------------


;directory_struc:
;		dq	0	; off:0   num. directories or num. files = num. records-1
;		dq	0	; off:8   fixed lenght = 128
;		dq	0	; off:16  total size of files (in level 3)
;		dq	0	; off:24  no used
;		dq	0	; off:32  level (1 or 2 for dir. only, 3 for files)
;		dq	0	; off:40   
;		dq	0	; off:48  
;		dq	0	; off:56
;		dq	0	; off:64
;		dq	0	; off:72
;		dq	0	; off:80
;		dq	0	; off:88
;		dq	0	; off:96
;		dq	0	; off:104
;		dq	0	; off:112 
;		dq	0	; off:120 end record 0

;file_struc:
;		NAME: times 72 db 0									; off:0   name(70), end string(1), tag(1)
;		dq	0										; off:72  size (bytes)
;		dq	0										; off:80  datetime
;		dq	0										; off:88  pointer to first cluster of pointers, lba
;		dq	0										; off:96  pointer to last  cluster of pointers, lba
;		dq	0										; off:104 not used
;		dq	0										; off:112 not used
;		dq	0										; off:120 not used

; pointers_struc:
;		dq	0	; off:0	 pointer to next cluster
;		dq	0	; off:8  pointer to previous cluster
;		dq	0	; off:16 first pointer to data [1]
;		...
;		...
;		...
;		...		; off:4072 last pointer to data (in this cluster) [508]
;		dq	0	; off:4080 zero terminated
;		dq	0	; off:4088 offset to first pointer to data to use

; ===================================================================================================
; EOF


