;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
;
; COMMAND LINE INTERFACE
;-----------------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; cli_input_string -- Take string from keyboard entry
;  IN:	RDI = location where string will be stored
;	RCX = number of characters to accept
; OUT:	RCX = length of string that was inputed (NULL not counted)
;	All other registers preserved
cli_input_string:
	push rdi
	push rsi
	push rdx			; Counter to keep track of max accepted characters
	push rbx
	push rax

	mov rbx, rdi			; store values for keycom
	mov ah, [screen_cursor_x]
	mov [cursorx], ah
	mov al, [screen_cursor_y]
	mov [cursory], al

	mov rdx, rcx
	xor rcx, rcx
.cli_input_string_more:
	call os_input_key_check
	jnc .cli_input_string_halt	; No key entered... halt until an interrupt is received
;---
	cmp al, 0			; Special key?
	jne .char

	cmp ah, 28			; If Enter key pressed, finish
	je .cli_input_string_done
	cmp ah, 14			; Backspace
	je .cli_input_string_backspace
	cmp ah, 72			; Arrow up
	je .cli_input_string_arrow_up
	cmp ah, 80			; Arrow down
	je .cli_input_string_arrow_down

	jmp .cli_input_string_more
;---
.char:
	cmp al, 32			; In ASCII range (32 - 126)?
	jb .cli_input_string_more
	cmp al, 126
	ja .cli_input_string_more
	cmp rcx, rdx			; Check if we have reached the max number of chars
	je .cli_input_string_more	; Jump if we have (should beep as well)
	stosb				; Store AL at RDI and increment RDI by 1
	inc rcx				; Increment the counter
	call os_print_char		; Display char
	jmp .cli_input_string_more

.cli_input_string_backspace:
	cmp rcx, 0			; backspace at the beginning? get a new char
	je .cli_input_string_more
	call os_dec_cursor		; Decrement the cursor
	mov al, 0x20			; 0x20 is the character for a space
	call os_print_char		; Write over the last typed character with the space
	call os_dec_cursor		; Decrement the cursor again
	dec rdi				; go back one in the string
	mov byte [rdi], 0x00		; NULL out the char
	dec rcx				; decrement the counter by one
	jmp .cli_input_string_more

.cli_input_string_halt:
	hlt				; Halt until another keystroke is received
	jmp .cli_input_string_more

.cli_input_string_done:
	mov al, 0x00
	stosb				; We NULL terminate the string


;---------------------------------------; Keycom : add last command
	mov rax, cli_key_com
	add rax, 9216
	xor rdx, rdx
.again:					; shift prev. commands
	mov rdi, rax			; ten slots
	sub rax, 1024			; 0  1  2  3  4  5  6  7  8  9
	mov rsi, rax			;                 	  |->x
	call os_string_copy		;  ... 		       |->x
	inc rdx
	cmp rdx, 9
	jne .again
				
	mov rdi, rbx			; restore value (rdi)
	mov rsi, rdi
	mov rdi, cli_key_com
	call os_string_copy		; copy last command

	mov [cli_key_com_counter], 0

;---------------------------------------; DEBUG
;	call os_print_newline
;	mov rax, cli_key_com
;	xor rdx, rdx
;.again2:	
;	mov rsi, rax
;	call os_print_string
;	call os_print_newline
;	add rax, 1024
;	inc rdx
;	cmp rdx, 10
;	jne .again2
;---------------------------------------;

	jmp .cli_input_string_end	; string_done finished and ret


;---------------------------------------; Keycom : search command up/down

.cli_input_string_arrow_up:
	cmp [cli_key_com_counter], 10	; counter 1 -> 10
	je .first

	inc [cli_key_com_counter]
	jmp .counter_ok

.first:
	mov [cli_key_com_counter], 1
	jmp .counter_ok


.cli_input_string_arrow_down:
	cmp [cli_key_com_counter], 0	; first time is zero
	je .last
	cmp [cli_key_com_counter], 1
	je .last

	dec [cli_key_com_counter]
	jmp .counter_ok
.last:
	mov [cli_key_com_counter], 10
	jmp .counter_ok


.counter_ok:
	mov al, [cli_key_com_counter]
	dec al				; offset start from 0
	mov rsi, cli_key_com

.pcom:					; point to the command
	cmp al, 0
	je .copy
	add rsi, 1024
	dec al
	jmp .pcom
	
.copy:				
	mov rdi, rbx			; restore value (rdi)
	call os_string_copy	
	mov ah, [cursorx]
	mov al, [cursory]
	call os_move_cursor
	call os_screen_clear_from_cursor
	call os_print_string
	call os_string_length
	add rdi, rcx

	jmp .cli_input_string_more
;---------------------------------------; Keycom : end


.cli_input_string_end:

	pop rax
	pop rbx
	pop rdx
	pop rsi
	pop rdi
	ret
; -----------------------------------------------------------------------------


os_command_line:
	call os_print_newline
	mov rsi, prompt			; Prompt for input
	mov rbx, ct_green		; Color
	call os_print_string_with_color
.input:
	mov rdi, cli_command_args_string
	mov rcx, 600			; Limit the input to 600 characters
	call cli_input_string		
	jrcxz .input			; cli_input_string stores the number of characters received in RCX

	mov rsi, cli_command_args_string
	call os_string_parse		; Remove extra spaces
	jrcxz .input			; os_string_parse stores the number of words in RCX

	mov byte [cli_args], cl		; Store the number of words in the string
	mov rsi, cli_command_args_string
	mov eax, dword[rsi]
	cmp eax, 'RUN '			; if the command is RUN...
	jne .break
	mov rdi, prg_path_args_string	; ...copy the string (command included)
	call os_string_copy
	mov al, [cli_args]		; set prg_args
	mov byte[prg_args], al

.break:
	; ------------------------------  Before clear the screen
	call os_screen_clear
	mov ax, 0x0001
	call os_move_cursor
	; ------------------------------

	; Break the contents of cli_command_args_string into individual strings
	mov rsi, cli_command_args_string
	mov al, 0x20
	mov bl, 0x00
	call os_string_change_char

	call os_string_uppercase	; Convert to uppercase for comparison

	mov rdi, cls_string		; 'CLS' entered?
	call os_string_compare
	jc near clear_screen

	mov rdi, ver_string		; 'VER' entered?
	call os_string_compare
	jc near print_ver

	mov rdi, date_string		; 'DATE' entered?
	call os_string_compare
	jc near date

	mov rdi, exit_string		; 'EXIT' entered?
	call os_string_compare
	jc near exit

	mov rdi, dev_string		; 'DEV' entered?
	call os_string_compare
	jc near dev

	mov rdi, mem_string		; 'MEM' entered?
	call os_string_compare
	jc near memory

	mov rdi, vbe_string		; 'VBE' entered?
	call os_string_compare
	jc near vbelist

	mov rdi, format_string		; 'FORMAT' entered?
	call os_string_compare
	jc near format_device

	mov rdi, help_string		; 'HELP' entered?
	call os_string_compare
	jc near print_help

	mov rdi, dir_string		; 'DIR' entered?
	call os_string_compare
	jc near dir

	mov rdi, mkd_string		; 'MKD' entered?
	call os_string_compare
	jc near mkdir

	mov rdi, mkf_string		; 'MKF' entered?
	call os_string_compare
	jc near mkfile

	mov rdi, ren_string		; 'REN' entered?
	call os_string_compare
	jc near rename

	mov rdi, del_string		; 'DEL' entered?
	call os_string_compare
	jc near delete

	mov rdi, cpf_string		; 'CPF' entered?
	call os_string_compare
	jc near copy

	mov rdi, mvf_string		; 'MVF' entered?
	call os_string_compare
	jc near move

	mov rdi, run_string		; 'RUN' entered?
	call os_string_compare
	jc near run

	mov rdi, keyb_string		; 'KEYB' entered?
	call os_string_compare
	jc near keyb

	mov rdi, tsmp_string		; 'TSMP' entered?
	call os_string_compare
	jc near tsmp

	mov rdi, time_string		; 'TIME' entered?
	call os_string_compare
	jc near time

	mov rdi, viewsector_string	; 'VS' entered?
	call os_string_compare
	jc near viewsector

	mov rdi, deletesector_string	; 'DS' entered?
	call os_string_compare
	jc near deletesector

	mov rdi, debug_string		; 'DEBUG' entered?
	call os_string_compare
	jc near debug

	mov rdi, reboot_string		; 'REBOOT' entered?
	call os_string_compare
	jc near reboot

	mov rdi, testzone_string	; 'TESTZONE' entered?
	call os_string_compare
	jc near testzone


fail:					; We didn't get a valid command
	mov rsi, not_found_msg
	call os_print_string
	jmp os_command_line		

;---------------------------------------------------------------------------------------------------

run:
	xor rcx, rcx
	mov cl, [cli_args]
	mov rsi, cli_command_args_string

	call fs_command
	jc os_command_line
	;--------------

	mov rax, programlocation	; 0x00600000
	xor rbx, rbx			; No arguments required (The program can get them with os_get_argc and os_get_argv)
	call os_smp_enqueue		; Queue the program to run on the next available core
	jmp exit			; The CLI can quit now. IRQ 8 will restart it when the program is finished

;--------------------------------------------------------------------------------------------------------------------------


dir:
mkdir:
mkfile:
rename:
delete:
	xor rcx, rcx
	mov cl, [cli_args]
	mov rsi, cli_command_args_string
	call fs_command

	jmp os_command_line

copy:
move:
	mov rsi, cli_command_args_string
	call fs_command2

	jmp os_command_line


format_device:
	mov rsi, msg_sure
	call os_print_string

	mov rdi, temp_str
	mov rcx, 1
	call os_input_string
	cmp byte[rdi], 'y'
	je .yes
	call os_print_newline
	jmp .end
.yes:
	call os_print_newline
	call fs_format
.end:
	jmp os_command_line

;-------------------------------------------------------------

print_help:
	mov rsi, help_text
	call os_print_string
	jmp os_command_line

clear_screen:
	call os_screen_clear
	mov ax, 0x0001
	call os_move_cursor
	jmp os_command_line

print_ver:
	mov rsi, version_msg
	call os_print_string
	jmp os_command_line

date:
	call os_get_date_string
	call os_print_string
	call os_print_newline
	jmp os_command_line

time:
	call os_get_time_string	
	call os_print_string
	call os_print_newline
	jmp os_command_line

keyb:
	xor eax, eax
	mov al, [cli_args]
	cmp al, 1
	jne .layout
;-------
	mov rsi, msg_keyb
	call os_print_string
.scancodes:
	xor rax, rax
	call os_input_key_wait
	mov bx, ax
	call os_print_char		; char
	mov rsi, space
	call os_print_string
	xor rax, rax
	mov al, bl
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string		; char code
	mov rsi, space
	call os_print_string
	xor rax, rax
	mov al, bh
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string		; scan code

	mov al, [screen_cursor_y]
	inc al
	cmp al, [nrows]			; rows
	jne .next_scancodes
	call os_screen_clear
	mov ax, 0x0001
	call os_move_cursor
	jmp keyb

.next_scancodes:
	call os_print_newline
	mov ah, bh
	cmp ah, 0x1C			; Enter
	jne .scancodes
	jmp .end

;-------
.layout:
	mov al, 1
	mov ah, 0
	mov rsi, cli_command_args_string
	call os_get_strv		; rsi = layout

	mov ax, word [rsi]
	cmp ax, 'IT'
	je .it
	cmp ax, 'US'
	je .us
	jmp .end

.it:
	mov [normal_keymap], normal_keymap_IT
	mov [shift_keymap], shift_keymap_IT
	mov [caplock_keymap], caplock_keymap_IT
	mov [alt_keymap], alt_keymap_IT
	mov [ctrl_keymap], ctrl_keymap_IT
	jmp .end
.us:
	mov [normal_keymap], normal_keymap_US
	mov [shift_keymap], shift_keymap_US
	mov [caplock_keymap], caplock_keymap_US
	mov [alt_keymap], alt_keymap_US
	mov [ctrl_keymap], ctrl_keymap_US
	;jmp .end
.end:
	jmp os_command_line

tsmp:
	mov rax, ap_print_id		; Our code to run on all CPUs
	xor rbx, rbx			; Clear RBX as there is no argument
	mov rcx, 64			; Number of instances to spawn

	spawn:
	call os_smp_enqueue
	sub rcx, 1
	cmp rcx, 0
	jne spawn

	bsp:
	call os_smp_dequeue		; Try to dequeue a workload
	jc emptyqueue			; If carry is set then the queue is empty
	call os_smp_run			; Otherwise run the workload
	jne bsp				; If it is not empty try to do another workload

	emptyqueue:
	call os_smp_wait		; Wait for all other processors to finish
	call os_print_newline


	jmp os_command_line


	; This procedure will be executed by each of the processors
	; It requires mutually exclusive access while it creates the string and prints to the screen
	; We must insure that only one CPU at a time can execute this code, so we employ a 'spinlock'.
	ap_print_id:
	mov rcx, 0x1FFFFF
	delay:
	dec rcx
	cmp rcx, 0
	jne delay

	grablock:
	bt word [mutex], 0		; Check if the mutex is free
	jnc grablock			; If not check it again

	lock				; The mutex was free, lock the bus
	btr word [mutex], 0		; Try to grab the mutex
	jnc grablock			; Jump if we were unsuccessful

	call os_smp_get_id		; Get the local APIC ID
	call os_debug_dump_al		; Print the APIC ID
	mov al, ' '
	call os_print_char

	bts word [mutex], 0		; Release the mutex
	ret
		

dev:
	xor eax, eax
	mov al, [cli_args]
	cmp al, 1
	jne .hddinfo			; If it is not only one then...
	
	mov rsi, device_list
	call os_print_string
	call os_print_newline
	jmp .end

.hddinfo:
	;------------------------------; DEBUG
	;mov al, byte [hd_ide_mask]	
	;mov edi, str_8digit
	;call os_byte_to_8digit_string
	;call os_print_string
	;mov rsi, space
	;call os_print_string
	;------------------------------

	mov al, 1
	mov ah, 0
	mov rsi, cli_command_args_string
	call os_get_strv		; rsi = device

	mov eax, dword[rsi]
	call current_device
	jc .end

	cmp dword [rsi], 'ata0'
	je .ata0
	cmp dword [rsi], 'ata1'
	je .ata1
	cmp dword [rsi], 'ata2'
	je .ata2
	cmp dword [rsi], 'ata3'
	je .ata3

	jmp .end	; type error

.ata0:
	mov r8, ata00_info
	jmp .print
.ata1:
	mov r8, ata01_info
	jmp .print
.ata2:
	mov r8, ata10_info
	jmp .print
.ata3:
	mov r8, ata11_info
	jmp .print


	; print model number
.print:
	mov rsi, str_name
	call os_print_string
	xor rbx, rbx
	xor rcx, rcx
.again:
	inc rbx
	mov al, [r8 + 54 + rbx]
	call os_print_char
	dec rbx
	mov al, [r8 + 54 + rbx] 	; 27*2
	call os_print_char
	add rbx, 2
	add rcx, 2
	cmp rcx, 40
	jne .again
	call os_print_newline
	;--------------------

	; general_configuration
	xor rax, rax
	mov ax, word [r8 + 0]		; 0

	bt ax, 15
	jnc .ata
	bt ax, 7
	jc .removable
	mov rsi, str_nodata
	jmp .print_gen
.ata:
	mov rsi, str_ata
	jmp .print_gen
.removable:
	mov rsi, str_removable
.print_gen:
	call os_print_string
	call os_print_newline
	;--------------------

	; lba_capacity
	mov rsi, str_lba28
	call os_print_string
	xor rax, rax
	mov eax, dword [r8 + 120] 	; 60*2
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string
	call os_print_newline
	;--------------------

	; max_lba_48bit
	mov rsi, str_lba48
	call os_print_string
	xor rax, rax
	mov rax, qword [r8 + 200]	; 100*2
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string
	call os_print_newline
	;--------------------

	mov rsi, [current.header]
	mov eax, dword[rsi]			; Signature
	cmp eax, 'BNFS'
	jne .end

	mov rsi, str_free
	call os_print_string
	mov rsi, [current.header]
	mov rax, [rsi + 16]			; Free Space Left (clusters)
	shl rax, 3				; mul 8 (sectors)
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string
	call os_print_newline

	mov rsi, str_nfiles
	call os_print_string
	mov rsi, [current.header]
	mov rax, [rsi + 24]			; Total number of files
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string
	call os_print_newline

	mov rsi, str_ndirs
	call os_print_string
	mov rsi, [current.header]
	mov rax, [rsi + 32]			; Total number of directories
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string
	call os_print_newline

.end:
	jmp os_command_line


memory:
	mov rsi, msg_memory_tab
	call os_print_string
	call os_print_newline

	mov rsi, 0x0000000000004000	; E820 Map location
.tablemem:
	mov rax, [rsi + 0]
	mov [base_m], rax
	mov rax, [rsi + 8]
	mov [lenght_m], rax
	mov eax, [rsi + 16]
	mov [type_m], eax
	
	cmp eax, 0			; if end
	je .end

	mov rcx, [lenght_m]
	cmp rcx, 0
	je .nextmem

	mov rax, [base_m]
	mov [end_m], rax
	add [end_m], rcx	
;------
	push rsi
	mov rax, [base_m]
	mov rdi, temp_str
	call os_int_to_hex_string
	mov rsi, temp_str
	call os_print_string
	call os_inc_cursor
	mov rax, [end_m]
	mov rdi, temp_str
	call os_int_to_hex_string
	mov rsi, temp_str
	call os_print_string
	call os_inc_cursor
	mov rax, [lenght_m]
	mov rdi, temp_str
	call os_int_to_hex_string
	mov rsi, temp_str
	call os_print_string
	call os_inc_cursor
	xor rax, rax
	mov eax, [type_m]
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string
	call os_print_newline
	pop rsi
;------
.nextmem:
	add rsi, 32
	jmp .tablemem
.end:
	jmp os_command_line


vbelist:
;-------
	mov rsi, [kVBE_info_ptr]	; use the original structure (from realmode)
	xor rax, rax
	mov eax, dword[rsi+14]
	call segoff_to_linear		; (modeptr>>12 & 0xFFFF0) + (modeptr & 0xFFFF)
	xor rbx, rbx
	mov ebx, eax			; address of modelist
;-------
	xor eax, eax
	mov al, [cli_args]
	cmp al, 1
	jne .mode

	; print list
.nextlist:
	xor rax, rax
	mov ax, word[rbx]
	cmp ax, 0xFFFF
	je .endlist
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string
	mov rsi, space
	call os_print_string

	add rbx, 2
	jmp .nextlist
.endlist:
	call os_print_newline

	; print default vesamode
	mov rsi, msg_default
	call os_print_string
	movzx rax, word[kVBE_vesamode]
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string
	call os_print_newline

	jmp .end
;-------

	; print selected modeinfo
.mode:
	mov al, 1
	mov ah, 0
	mov rsi, cli_command_args_string
	call os_get_strv		; rsi = mode (str)
	call os_string_to_int		; rax = mode (int)
	mov rdx, rax			; save value
	xor rcx, rcx			; counter (offset)

.nextlist2:
	xor rax, rax
	mov ax, word[rbx]
	cmp ax, 0xFFFF
	je .end				; not found

	cmp ax, dx
	je .endlist2			; found

	add rbx, 2
	inc rcx
	jmp .nextlist2
.endlist2:

	mov rax, 66+2
	mul rcx
	mov rsi, 0x20000 		; table address
	add rsi, rax			; + offset

	mov rbx, msg_vbe01
	movzx rax, word[rsi+0]
	call print_ns
	mov rbx, msg_vbe02
	movzx rax, byte[rsi+2]
	call print_ns
	mov rbx, msg_vbe03
	movzx rax, byte[rsi+3]
	call print_ns
	mov rbx, msg_vbe04
	movzx rax, word[rsi+4]
	call print_ns
	mov rbx, msg_vbe05
	movzx rax, word[rsi+6]
	call print_ns
	mov rbx, msg_vbe06
	movzx rax, word[rsi+8]
	call print_ns
	mov rbx, msg_vbe07
	movzx rax, word[rsi+10]
	call print_ns
	mov rbx, msg_vbe08
	xor rax, rax
	mov eax, dword[rsi+12]
	call print_ns
	mov rbx, msg_vbe09
	movzx rax, word[rsi+16]
	call print_ns

	mov rbx, msg_vbe10
	movzx rax, word[rsi+18]
	call print_ns
	mov rbx, msg_vbe11
	movzx rax, word[rsi+20]
	call print_ns

	mov rbx, msg_vbe12
	movzx rax, byte[rsi+22]
	call print_ns
	mov rbx, msg_vbe13
	movzx rax, byte[rsi+23]
	call print_ns
	mov rbx, msg_vbe14
	movzx rax, byte[rsi+24]
	call print_ns
	mov rbx, msg_vbe15
	movzx rax, byte[rsi+25]
	call print_ns
	mov rbx, msg_vbe16
	movzx rax, byte[rsi+26]
	call print_ns
	mov rbx, msg_vbe17
	movzx rax, byte[rsi+27]
	call print_ns
	mov rbx, msg_vbe18
	movzx rax, byte[rsi+28]
	call print_ns
	mov rbx, msg_vbe19
	movzx rax, byte[rsi+29]
	call print_ns
	mov rbx, msg_vbe20
	movzx rax, byte[rsi+30]
	call print_ns

	mov rbx, msg_vbe21
	movzx rax, byte[rsi+31]
	call print_ns
	mov rbx, msg_vbe22
	movzx rax, byte[rsi+32]
	call print_ns
	mov rbx, msg_vbe23
	movzx rax, byte[rsi+33]
	call print_ns
	mov rbx, msg_vbe24
	movzx rax, byte[rsi+34]
	call print_ns
	mov rbx, msg_vbe25
	movzx rax, byte[rsi+35]
	call print_ns
	mov rbx, msg_vbe26
	movzx rax, byte[rsi+36]
	call print_ns
	mov rbx, msg_vbe27
	movzx rax, byte[rsi+37]
	call print_ns
	mov rbx, msg_vbe28
	movzx rax, byte[rsi+38]
	call print_ns
	mov rbx, msg_vbe29
	movzx rax, byte[rsi+39]
	call print_ns

	mov rbx, msg_vbe30
	xor rax, rax
	mov eax, dword[rsi+40]
	call print_ns
	mov rbx, msg_vbe31
	xor rax, rax
	mov eax, dword[rsi+44]
	call print_ns
	mov rbx, msg_vbe32
	movzx rax, word[rsi+48]
	call print_ns

;------end vbe 2.0

	mov rbx, msg_vbe33
	movzx rax, word[rsi+50]
	call print_ns
	mov rbx, msg_vbe34
	movzx rax, byte[rsi+52]
	call print_ns
	mov rbx, msg_vbe35
	movzx rax, byte[rsi+53]
	call print_ns
	mov rbx, msg_vbe36
	movzx rax, byte[rsi+54]
	call print_ns
	mov rbx, msg_vbe37
	movzx rax, byte[rsi+55]
	call print_ns
	mov rbx, msg_vbe38
	movzx rax, byte[rsi+56]
	call print_ns
	mov rbx, msg_vbe39
	movzx rax, byte[rsi+57]
	call print_ns
	mov rbx, msg_vbe40
	movzx rax, byte[rsi+58]
	call print_ns
	mov rbx, msg_vbe41
	movzx rax, byte[rsi+59]
	call print_ns
	mov rbx, msg_vbe42
	movzx rax, byte[rsi+60]
	call print_ns
	mov rbx, msg_vbe43
	movzx rax, byte[rsi+61]
	call print_ns
	mov rbx, msg_vbe44
	xor rax, rax
	mov eax, dword[rsi+62]
	call print_ns
	call os_print_newline
.end:
	jmp os_command_line


deletesector:
	mov rsi, msg_sure
	call os_print_string

	mov rdi, temp_str
	mov rcx, 1
	call os_input_string
	cmp byte[rdi], 'y'
	je .ds
	call os_print_newline
	jmp .end

.ds:
	xor eax, eax
	mov al, [cli_args]
	cmp al, 3
	jne .error

	mov al, 1
	mov ah, 0
	mov rsi, cli_command_args_string
	call os_get_strv		; rsi = device

	mov eax, dword [rsi]	
	call current_device
	jc .error

	mov al, 2
	mov ah, 0
	mov rsi, cli_command_args_string
	call os_get_strv		; rsi = LBA sector
	call os_string_to_int		; rax

	mov rbx, [current.base]
	mov rdx, [current.id]
	mov rcx, 1
	mov rsi, sector0
	call [current.writesectors]

	cmp rcx, 0
	je .error

	call os_print_newline
	jmp .end

.error:
	mov rsi, msg_error
	call os_print_string
	call os_print_newline
.end:
	jmp os_command_line


viewsector:
	xor eax, eax
	mov al, [cli_args]
	cmp al, 3
	jne .error

	mov al, 1
	mov ah, 0
	mov rsi, cli_command_args_string
	call os_get_strv		; rsi = device

	mov eax, dword [rsi]	
	call current_device
	jc .error

	mov al, 2
	mov ah, 0
	mov rsi, cli_command_args_string
	call os_get_strv		; rsi = LBA sector
	call os_string_to_int		; rax

	mov rbx, [current.base]
	mov rdx, [current.id]
	mov rcx, 1
	mov rdi, sector2
	call [current.readsectors]

	cmp rcx, 0
	je .error

	call os_print_newline

	mov rsi, sector2
	mov rcx, 512
	call os_debug_dump_mem

	call os_print_newline
	jmp .end

.error:
	mov rsi, msg_error
	call os_print_string
	call os_print_newline
.end:
	jmp os_command_line



debug:					; debug <startmem> <amount>
	xor eax, eax
	mov al, [cli_args]
	cmp al, 1
	je .debug_dump_reg		; If it is only one then do a register dump
	mov rcx, 16	
	cmp al, 3			; Did we get at least 3?
	jb .noamount			; If not no amount was specified
	mov al, 2
	mov ah, 0
	mov rsi, cli_command_args_string
	call os_get_strv		; Get the amount of bytes to display
	call os_string_to_int		; Convert to an integer
	mov rcx, rax
.noamount:
	mov ah, 0
	mov al, 1
	mov rsi, cli_command_args_string
	call os_get_strv		; Get the starting memory address
	call os_hex_string_to_int
	mov rsi, rax
.debug_default:
	call os_debug_dump_mem
	call os_print_newline
	jmp os_command_line

.debug_dump_reg:
	call os_debug_dump_reg
	jmp os_command_line


reboot:
	in al, 0x64
	test al, 00000010b		; Wait for an empty Input Buffer
	jne reboot
	mov al, 0xFE
	out 0x64, al			; Send the reboot call to the keyboard controller
	jmp reboot


testzone:				; used during development

	jmp os_command_line


exit:
	ret



;------------------------------------------------------------------------------------------------------------
; Strings
;------------------------------------------------------------------------------------------------------------
	help_text		db 'Built-in commands:', 10
				db 'VER, CLS, DATE, TIME, HELP, DEV, MEM, REBOOT', 10
				db 'DIR, MKD, MKF, REN, DEL, CPF, MVF, RUN, FORMAT', 10
				db 'KEYB, VBE, TSMP, DEBUG, VS, DS' , 10, 0
	not_found_msg		db 'Command not found.', 10, 0
	version_msg		db 'BareNumbersOS v0.3 - 2012 - (GPL v.3)', 10, 0

	help_string		db 'HELP', 0
	cls_string		db 'CLS', 0
	ver_string		db 'VER', 0
	date_string		db 'DATE', 0
	time_string		db 'TIME', 0
	dev_string		db 'DEV', 0
	mem_string		db 'MEM', 0
	vbe_string		db 'VBE', 0
	format_string		db 'FORMAT', 0
	dir_string		db 'DIR', 0
	mkd_string		db 'MKD',0
	mkf_string		db 'MKF',0
	ren_string		db 'REN', 0
	del_string		db 'DEL', 0
	cpf_string		db 'CPF',0
	mvf_string		db 'MVF',0
	run_string		db 'RUN', 0
	keyb_string		db 'KEYB', 0
	tsmp_string		db 'TSMP', 0
	viewsector_string	db 'VS', 0
	deletesector_string	db 'DS', 0
	debug_string		db 'DEBUG', 0
	reboot_string		db 'REBOOT', 0
	exit_string		db 'EXIT', 0
	testzone_string		db 'TESTZONE', 0

;------------
; Device info
;------------
	str_name		db 'Name  : ',0
	str_ata 		db 'Gener.: ATA',0
	str_removable 		db 'Gener.: REMOVABLE',0
	str_nodata 		db 'Gener.: NO DATA',0
	str_lba28		db 'LBA28 : ',0
	str_lba48		db 'LBA48 : ',0

	str_free		db 'Free  : ',0
	str_nfiles		db 'Files : ',0
	str_ndirs		db 'Dirs  : ',0

; =============================================================================
; EOF
