;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
; Initializations
;-----------------------------------------------------------------------------------

K_Init:

	; Set up RTC
	call InitRTC

	; Set PIT Channel 0 to fire at 100Hz (Divisor = 1193180 / hz)
	mov al, 0x34			; 0x34 = 00110100b
	out 0x43, al
	mov al, 0x9B			; lsb - We want 100Hz so 0x2E9B
	out 0x40, al
	mov al, 0x2E			; msb - 1.19318Mhz/11931 = 100 Hz
	out 0x40, al


	; Set keyboard repeat rate to max
	mov al, 0xf3
	out 0x60, al			; Set Typematic Rate/Delay
	xor al, al
	out 0x60, al			; 30 cps and .25 second delay

	; Set keyboard's leds
	mov [KeyboardStatus], 0
	call SetKeyBoardLeds

	; Empty the keyboard's buffer
	xor cx, cx          		; Allow 65,536 times thru loop.
.Wait4Empty:
     	in  al, 64h         		; Read keyboard status register.
        test al, 10b         		; Input buffer full?
        loopnz .Wait4Empty      	; If so, wait until empty.

	; Disable blink
	mov dx, 0x3DA
	in al, dx
	mov dx, 0x3C0
	mov al, 0x30
	out dx, al
	add dx, 1
	in al, dx
	and al, 0xF7
	sub dx, 1
	out dx, al

	; Set keyboard layout
	;US
	mov [normal_keymap], normal_keymap_US
	mov [shift_keymap], shift_keymap_US
	mov [caplock_keymap], caplock_keymap_US
	mov [alt_keymap], alt_keymap_US
	mov [ctrl_keymap], ctrl_keymap_US
	;IT
	;mov [normal_keymap], normal_keymap_IT
	;mov [shift_keymap], shift_keymap_IT
	;mov [caplock_keymap], caplock_keymap_IT
	;mov [alt_keymap], alt_keymap_IT
	;mov [ctrl_keymap], ctrl_keymap_IT

	; Enable specific interrupts
	;---------------------------
	in al, 0x21
	mov al, 11111000b		; Enable Timer (0), Keyboard (1), Cascade (2)
	out 0x21, al
	in al, 0xA1
	mov al, 11111110b		; Enable RTC (8)
	out 0xA1, al

ret


; ===================================================================================================
; EOF

