;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
; VBE tables
;
;-----------------------------------------------------------------------------------


;---------------------------------------------------
;  VBE ModeInfoBlock
;---------------------------------------------------


struc VBE_MODE_INFOBLOCKSTRUCTURE 
	{

	;general fields
	;---------------

      	.modeattr 	dw 0    	;  0 : Mode attributes
      	.wina  		db 0     	;  2 : Window A attributes
      	.winb  		db 0     	;  3 : Window B attributes
      	.gran  		dw 0     	;  4 : Window granularity in KiB
      	.winsize	dw 0     	;  6 : Window size in KiB
      	.sega  		dw 0     	;  8 : Start segment of window A
      	.segb  		dw 0     	; 10 : Start segment of window B        
      	.winfunc_adr 	dd 0    	; 12 : Pointer FAR (real mode) to window positioning function (equ set bank: AX=4F05h)
      	.bytesperline 	dw 0		; 16 : Bytes per scan line

        ;---remainder is optional for VESA modes, needed for OEM modes---
        .xres 		dw 0   		; 18 : Width in pixels
        .yres	 	dw 0  		; 20 : Height in pixels
        .charw 		db 0   		; 22 : Width of character cell in pixels
        .charh 		db 0   		; 23 : Height of character cell in pixels
        .planes 	db 0   		; 24 : Number of memory planes
        .bpp  		db 0      	; 25 : Number of bits per pixel
        .banks	 	db 0     	; 26 : Number of banks
        .mmodel 	db 0      	; 27 : Memory model type
        .banksize	db 0      	; 28 : Size of bank in KiB
        .images 	db 0     	; 29 : Number of image pages
        .reserved   	db 0        	; 30 : Reserved(1)

        ;------VBE v1.2+ ------ Direct Color fields (required for direct/6 and YUV/7 memory models)
        .redmasksize 	db 0       	; 31 : Red mask - size of direct color (#bits)
        .redfieldpos 	db 0       	; 32 : Red mask - bit position of lsb
        .greenmasksize 	db 0     	; 33 : Green mask - size of direct color (#bits)
        .greenfieldpos 	db 0     	; 34 : Green mask - bit position of lsb
        .bluemasksize 	db 0      	; 35 : Blue mask - size of direct color (#bits)
        .bluefieldpos 	db 0      	; 36 : Blue mask - bit position of lsb
        .rsvdmasksize 	db 0      	; 37 : Reserved mask - size of direct color (#bits)
        .rsvdfieldpos 	db 0      	; 38 : Reserved mask - bit position of lsb
        .dcolorinf   	db 0     	; 39 : DirectColorModeInfo

        ;------VBE v2+ ------------------------------------------------------------------- 
        .LFB	 	dd 0       	; 40 : Physical address of linear video buffer
        .offscreen	dd 0     	; 44 : Offset (pointer) to start of offscreen memory
        .offsize	dw 0     	; 48 : Size of offscreen memory in KiB

	;-- Mandatory information for VBE 3.0 and above-----------------------------------
        .linbytesperline   dw 0      	; 50 : Bytes per scan line (linear modes)
        .bankedpages 	   db 0       	; 52 : Number of pages for banked modes
        .linearpages 	   db 0       	; 53 : Number of pages for linear modes
        .linredmasksize    db 0       	; 54 : Red mask - size of direct color (linear modes)
        .linredfieldpos    db 0       	; 55 : Red mask - bit position of lsb (linear modes)
        .lingreenmasksize  db 0     	; 56 : Green mask - size of direct color (linear modes)
        .lingreenfieldpos  db 0     	; 57 : Green mask - bit position of lsb (linear modes)
        .linbluemasksize   db 0      	; 58 : Blue mask - size of direct color  (linear modes)
        .linbluefieldpos   db 0      	; 59 : Blue mask - bit position of lsb (linear modes)
        .linrsvdmasksize   db 0      	; 60 : Reserved mask - size of direct color (linear modes)
        .linrsvdfieldpos   db 0      	; 61 : Reserved mask - bit position of lsb (linear modes)
        .maxpixelclock 	   dd 0     	; 62 : Maximum pixel clock (in Hz) for graphics mode
        .OEMbuffer: 	times 190 db 0  ; 66 : OEM scratchpad

	};end of structure (256 bytes)

;-------------------------------------------------------------------
;  VBE Info Block
;-------------------------------------------------------------------

struc VBE_INFOBLOCKSTRUCTURE 
	{
	;general fields
	;---------------

        .sign 		dd 0      	;  0 : Signature
        .vers 		dw 0      	;  4 : Vesa version number
        .oem	 	dd 0    	;  6 : Pointer to OEM name
        .capab 		dd 0     	; 10 : Capabilities
        .modelist 	dd 0  		; 14 : Pointer to list of supported VESA and
                       			;      OEM video modes Terminated with 0FFFFh
        .memory  	dw 0   		; 18 : Video memory in 64 KiB blocks

	;Remaining fields are for VBE 2.0 or later
	;------------------------------------------

        .oemver 	dw 0   		; 20 : OEM version
        .vendor 	dd 0   		; 22 : Pointer to vendor name
        .product	dd 0  		; 26 : Pointer to product name
        .prodrev	dd 0    	; 30 : Pointer to product revision
	.reserved:	times 222 db 0  ; 34 : Reserved for VBE
	.OEMdata:	times 256 db 0  ;256 : OEM scratchpad

	};end of structure (512 bytes)

align 16
         vesa_info 		VBE_INFOBLOCKSTRUCTURE        	; Structure for VBE info block.
         vbe		 	VBE_MODE_INFOBLOCKSTRUCTURE	; Structure for obtaining specific mode info.


;------------------------------------------------------------


; ===================================================================================================
; EOF

;--------------------------------------------------------
;  VBE Info Block
;--------------------------------------------------------
;
; Capabilities
;
; VBE_CAPABILITY_8BIT_DAC                          0x0001
; VBE_CAPABILITY_NOT_VGA_COMPATIBLE                0x0002
; VBE_CAPABILITY_RAMDAC_USE_BLANK_BIT              0x0004
; VBE_CAPABILITY_STEREOSCOPIC_SUPPORT              0x0008
; VBE_CAPABILITY_STEREO_VIA_VESA_EVC               0x0010
;
; Notes:
; All pointers are real mode seg:off types.
; The version word is BCD coded. VBE 2.0 is 200h, VBE 1.2 is 102h.
; Under VBE 1.x the buffer is 256 bytes. (Some VBE 1.x BIOS's are
; buggy so you should always use 512 bytes)
; Under VBE 2.0 the buffer is 512 bytes.
; If Signature is 'VBE2' before the call then VBE2.0 info is
; returned. Signature will always be set to 'VESA' after.
;
; Capabilities is as follows:
; bit 0 = 0     DAC is fixed width, with 6 bits per primary colour
;       = 1     DAC width is switchable to 8 bits per primary colour
; bit 1 = 0     Controller is VGA compatible
;       = 1     Controller is not VGA compatible
; bit 2 = 0     Normal RAMDAC operation
;       = 1     When programming large blocks of information to the
;               RAMDAC use blank bit in Function 09h.
;
; When ever you set a mode the DAC is always set to 6 bits/primary colour

;--------------------------------------------------------
;  VBE ModeInfoBlock
;--------------------------------------------------------
;
; Mode Attributes
;
; VBE_MODE_ATTRIBUTE_SUPPORTED                     0x0001
; VBE_MODE_ATTRIBUTE_EXTENDED_INFORMATION_AVAIL    0x0002
; VBE_MODE_ATTRIBUTE_TTY_BIOS_SUPPORT              0x0004
; VBE_MODE_ATTRIBUTE_COLOR_MODE                    0x0008
; VBE_MODE_ATTRIBUTE_GRAPHICS_MODE                 0x0010
; VBE_MODE_ATTRIBUTE_NOT_VGA_COMPATIBLE            0x0020
; VBE_MODE_ATTRIBUTE_NO_VGA_COMPATIBLE_WINDOW      0x0040
; VBE_MODE_ATTRIBUTE_LINEAR_FRAME_BUFFER_MODE      0x0080
; VBE_MODE_ATTRIBUTE_DOUBLE_SCAN_MODE              0x0100
; VBE_MODE_ATTRIBUTE_INTERLACE_MODE                0x0200
; VBE_MODE_ATTRIBUTE_HARDWARE_TRIPLE_BUFFER        0x0400
; VBE_MODE_ATTRIBUTE_HARDWARE_STEREOSCOPIC_DISPLAY 0x0800
; VBE_MODE_ATTRIBUTE_DUAL_DISPLAY_START_ADDRESS    0x1000
;
; Window attributes
;
; VBE_WINDOW_ATTRIBUTE_RELOCATABLE                 0x01
; VBE_WINDOW_ATTRIBUTE_READABLE                    0x02
; VBE_WINDOW_ATTRIBUTE_WRITEABLE                   0x04
;
; Memory model
;
; VBE_MEMORYMODEL_TEXT_MODE                        0x00
; VBE_MEMORYMODEL_CGA_GRAPHICS                     0x01
; VBE_MEMORYMODEL_HERCULES_GRAPHICS                0x02
; VBE_MEMORYMODEL_PLANAR                           0x03
; VBE_MEMORYMODEL_PACKED_PIXEL                     0x04
; VBE_MEMORYMODEL_NON_CHAIN_4_256                  0x05
; VBE_MEMORYMODEL_DIRECT_COLOR                     0x06
; VBE_MEMORYMODEL_YUV                              0x07
;
; DirectColorModeInfo
;
; VBE_DIRECTCOLOR_COLOR_RAMP_PROGRAMMABLE          0x01
; VBE_DIRECTCOLOR_RESERVED_BITS_AVAILABLE          0x02
;
;------------------------------------------------------
; Notes:
;
;(extra)
; WinGranularity (banking mode only)
; = 64 the banks do not overlap
; = 32 the banks overlap 50% of each other
; = 16 then they overlap 75% of each other, etc...
; Colour formats
; VBE Version 1.1 and earlier defined Direct Colour graphics modes with
; pixel formats 1:5:5:5, 8:8:8, and 8:8:8:8  as a Packed Pixel model
; with 16, 24, and 32 bits per pixel, respectively.
; In VBE Version 1.2 and later, the Direct Colour modes use the
; Direct Colour memory model and use the MaskSize and FieldPosition fields
; of the ModeInfoBlock to describe the pixel format.
; BytesperScanLine
; The BytesPerScanLine field specifies how many full bytes are in each
; logical scanline. The logical scanline could be equal to or larger
; than the displayed scanline. Therefore you can not assume an image is a
; continuous stream of data in VRAM. It could have extra (ignored) data at
; the end of each scan line. But this data could be visible if you use
; func #7 (set/get display start)
; NumberImagePages
; This indicates how many full images can be loaded
; into VRAM and then each could be displayed one at a time. (good for
; flicker-free animations). ie: 0 means 1 image, 1 means 2 images, etc.
;
;-------------------------------------------------------------------------
; From VBE20.txt Core Functions Standard
;
;     The ModeAttributes field describes certain important characteristics
;     of the graphics mode.
;
;     The ModeAttributes field is defined as follows:
;
;     D0   =    Mode supported by hardware configuration
;          0 =  Mode not supported in hardware
;          1 =  Mode supported in hardware
;     D1   =    1 (Reserved)
;     D2   =    TTY Output functions supported by BIOS
;          0 =  TTY Output functions not supported by BIOS
;          1 =  TTY Output functions supported by BIOS
;     D3   =    Monochrome/color mode (see note below)
;          0 =  Monochrome mode
;          1 =  Color mode
;     D4   =    Mode type
;          0 =  Text mode
;          1 =  Graphics mode
;     D5   =    VGA compatible mode
;          0 =  Yes
;          1 =  No
;     D6   =    VGA compatible windowed memory mode is available
;          0 =  Yes
;          1 =  No
;     D7   =    Linear frame buffer mode is available
;          0 =  No
;         1 =   Yes
;     D8-D15 =  Reserved
;
;     Bit D0 is set to indicate that this mode can be initialized in the
;     present hardware configuration. This bit is reset to indicate the
;     unavailability of a graphics mode if it requires a certain monitor
;     type, more memory than is physically installed, etc.
;
;     Bit D1 was used by VBE 1.0 and 1.1 to indicate that the optional
;     information following the BytesPerScanLine field were present in the
;     data structure.  This information became mandatory with VBE version
;     1.2 and above, so D1 is no longer used and should be set to 1.  The
;     Direct Color fields are valid only if the MemoryModel field is set to
;     a 6 (Direct Color) or 7 (YUV).
;     Bit D2 indicates whether the video BIOS has support for output
;     functions like TTY output, scroll, etc. in this mode.  TTY support is
;     recommended but not required for all extended text and graphic modes.
;     If bit D2 is set to 1, then the INT 10h BIOS must support all of the
;     standard output functions listed below.
;
;     All of the following TTY functions must be supported when this bit is
;     set:
;
;          01   Set Cursor Size
;          02   Set Cursor Position
;          06   Scroll TTY window up or Blank Window
;          07   Scroll TTY window down or Blank Window
;          09   Write character and attribute at cursor position
;          0A   Write character only at cursor position
;          0E   Write character and advance cursor  
;
;     Bit D3 is set to indicate color modes, and cleared for monochrome
;     modes.
;
;     Bit D4 is set to indicate graphics modes, and cleared for text modes.
;
;     Note: Monochrome modes map their CRTC address at 3B4h. Color modes map
;     their CRTC address at 3D4h. Monochrome modes have attributes in which
;     only bit 3 (video) and bit 4 (intensity) of the attribute controller
;     output are significant. Therefore, monochrome text modes have
;     attributes of off, video, high intensity, blink, etc. Monochrome
;     graphics modes are two plane graphics modes and have attributes of
;     off, video, high intensity, and blink. Extended two color modes that
;     have their CRTC address at 3D4h, are color modes with one bit per
;     pixel and one plane. The standard VGA modes, 06h and 11h would be
;     classified as color modes, while the standard VGA modes 07h and 0Fh
;     would be classified as monochrome modes.
;
;     Bit D5 is used to indicate if the mode is compatible with the VGA
;     hardware registers and I/O ports.  If this bit is set, then the mode
;     is NOT VGA compatible and no assumptions should be made about the
;     availability of any VGA registers.   If clear, then the standard VGA
;     I/O ports and frame buffer address defined in WinASegment and/or
;     WinBSegment can be assumed.
;
;     Bit D6 is used to indicate if the mode provides Windowing or Banking
;     of the frame buffer into the frame buffer memory region specified by
;     WinASegment and WinBSegment.  If set, then Windowing of the frame
;     buffer is NOT possible.  If clear, then the device is capable of
;     mapping the frame buffer into the segment specified in WinASegment
;     and/or WinBSegment.  (This bit is used in conjunction with bit D7, see
;     table following D7 for usage)
;
;     Bit D7 indicates the presence of a Linear Frame Buffer memory model.
;     If this bit is set, the display controller can be put into a flat
;     memory model by setting the mode (VBE Function 02h) with the Flat
;     Memory Model bit set. (This bit is used in conjunction with bit D6,
;     see following table for usage)
;
;                      +-----------+-----+-----+
;                      |           | D7  | D6  |
;                      +-----------+-----+-----+
;                      |  Windowed |  0  |  0  |
;                      |   frame   |     |     |
;                      |   buffer  |     |     |
;                      |    only   |     |     |
;                      +-----------+-----+-----+
;                      |    n/a    |  0  |  1  |
;                      +-----------+-----+-----+
;                      |    Both   |  1  |  0  |
;                      |  Windowed |     |     |
;                      |    and 4  |     |     |
;                      |  Linear   |     |     |
;                      +-----------+-----+-----+
;                      |   Linear  |  1  |  1  |
;                      |   frame   |     |     |
;                      |   buffer  |     |     |
;                      |    only   |     |     |
;                      +-----------+-----+-----+
;
;
;     The BytesPerScanLine field specifies how many full bytes are in each
;     logical scanline. The logical scanline could be equal to or larger
;     than the displayed scanline.
;
;     The WinAAttributes and WinBAttributes describe the characteristics of
;     the CPU windowing scheme such as whether the windows exist and are
;     read/writeable, as follows:
;
;     D0    =    Relocatable window(s) supported
;           0 =  Single non-relocatable window only
;           1 =  Relocatable window(s) are supported
;     D1    =    Window readable
;           0 =  Window is not readable
;           1 =  Window is readable
;     D2    =    Window writeable
;           0 =  Window is not writeable
;           1 =  Window is writeable
;     D3-D7 =    Reserved
;
;     Even if windowing is not supported, (bit D0 = 0 for both Window A and
;     Window B), then an application can assume that the display memory
;     buffer resides at the location specified by WinASegment and/or
;     WinBSegment.
;
;     WinGranularity specifies the smallest boundary, in KB, on which the
;     window can be placed in the frame buffer memory. The value of this
;     field is undefined if Bit D0 of the appropriate WinAttributes field is
;     not set.
;
;     WinSize specifies the size of the window in KB.
;
;     WinASegment and WinBSegment address specify the segment addresses
;     where the windows are located in the CPU address space.
;
;     Use D14 of the Mode Number to select the Linear Buffer on a mode set
;     (Function 02h).
;
;     WinFuncPtr specifies the segment:offset of the VBE memory windowing
;     function. The windowing function can be invoked either through VBE
;     Function 05h, or by calling the function directly. A direct call will
;     provide faster access to the hardware paging registers than using VBE
;     Function 05h, and is intended to be used by high performance
;     applications. If this field is NULL, then VBE Function 05h must be
;     used to set the memory window when paging is supported.  This direct
;     call method uses the same parameters as VBE Function 05h including AX
;     and for VBE 2.0 implementations will return the correct Return Status.
;     VBE 1.2 implementations and earlier, did not require the Return Status
;     information to be returned.  For more information on the direct call
;     method, see the notes in VBE Function 05h and the sample code in
;     Appendix 5.
;
;     The XResolution and YResolution specify the width and height in pixel
;     elements or characters for this display mode. In graphics modes, these
;     fields indicate the number of horizontal and vertical pixels that may
;     be displayed. In text modes, these fields indicate the number of
;     horizontal and vertical character positions.  The number of pixel
;     positions for text modes may be calculated by multiplying the returned
;     XResolution and YResolution values by the character cell width and
;     height indicated in the XCharSize and YCharSize fields described
;     below.
;
;     The XCharSize and YCharSize specify the size of the character cell in
;     pixels.  (This value is not zero based)  e.g. XCharSize for Mode 3
;     using the 9 point font will have a value of 9.
;
;     The NumberOfPlanes field specifies the number of memory planes
;     available to software in that mode. For standard 16-color VGA
;     graphics, this would be set to 4. For standard packed pixel modes, the
;     field would be set to 1.  For 256-color non-chain-4 modes, where you
;     need to do banking to address all pixels this value should be set to
;     the number of banks required to get to all the pixels (typically this
;     will be 4 or 8).
;
;     The BitsPerPixel field specifies the total number of bits allocated to
;     one pixel. For example, a standard VGA 4 Plane 16-color graphics mode
;     would have a 4 in this field and a packed pixel 256-color graphics
;     mode would specify 8 in this field. The number of bits per pixel per
;     plane can normally be derived by dividing the BitsPerPixel field by
;     the NumberOfPlanes field.
;
;     The MemoryModel field specifies the general type of memory
;     organization used in this mode. The following models have been
;     defined:
;
;     00h  =    Text mode
;     01h  =    CGA graphics
;     02h  =    Hercules graphics
;     03h  =    Planar
;     04h  =    Packed pixel
;     05h  =    Non-chain 4, 256 color
;     06h  =    Direct Color
;     07h  =    YUV
;     08h-0Fh =  Reserved, to be defined by VESA
;     10h-FFh =  To be defined by OEM
;
;     VBE Version 1.1 and earlier defined Direct Color graphics modes with
;     pixel formats 1:5:5:5, 8:8:8, and 8:8:8:8  as a Packed Pixel model
;     with 16, 24, and 32 bits per pixel, respectively. In VBE Version 1.2
;     and later, the Direct Color modes use the Direct Color memory model
;     and use the MaskSize and FieldPosition fields of the ModeInfoBlock to
;     describe the pixel format. BitsPerPixel is always defined to be the
;     total memory size of the pixel, in bits.
;
;     NumberOfBanks. This is the number of banks in which the scan lines are
;     grouped. The quotient from dividing the scan line number by the number
;     of banks is the bank that contains the scan line and the remainder is
;     the scan line number within the bank. For example, CGA graphics modes
;     have two banks and Hercules graphics mode has four banks. For modes
;     that don't have scanline banks (such as VGA modes 0Dh-13h), this field
;     should be set to 1.
;
;     The BankSize field specifies the size of a bank (group of scan lines)
;     in units of 1 KB. For CGA and Hercules graphics modes this is 8, as
;     each bank is 8192 bytes in length. For modes that don't have scanline
;     banks (such as VGA modes 0Dh-13h), this field should be set to 0.
;
;     The NumberOfImagePages field specifies the "total number minus one (-
;     1)"of complete display images that will fit into the frame buffer
;     memory. The application may load more than one image into the frame
;     buffer memory if this field is non-zero, and move the display window
;     within each of those pages.  This should only be used for determining
;     the additional display pages which are available to the application;
;     to determine the available off screen memory, use the
;     OffScreenMemOffset and OffScreenMemSize information.
;
;     Note: If the ModeInfoBlock is for an IBM Standard VGA mode and the
;     NumberOfImagePages field contains more pages than would be found in a
;     256KB implementation, the TTY support described in the ModeAttributes
;     must be accurate.  i.e. if the TTY functions are claimed to be
;     supported, they must be supported in all pages, not just the pages
;     normally found in the 256KB implementation.
;
;     The Reserved field has been defined to support a future VBE feature
;     and will always be set to one in this version.
;
;     The RedMaskSize, GreenMaskSize, BlueMaskSize, and RsvdMaskSize fields
;     define the size, in bits, of the red, green, and blue components of a
;     direct color pixel. A bit mask can be constructed from the MaskSize
;     fields using simple shift arithmetic. For example, the MaskSize values
;     for a Direct Color 5:6:5 mode would be 5, 6, 5, and 0, for the red,
;     green, blue, and reserved fields, respectively. Note that in the YUV
;     MemoryModel, the red field is used for V, the green field is used for
;     Y, and the blue field is used for U. The MaskSize fields should be set
;     to 0 in modes using a memory model that does not have pixels with
;     component fields.
;
;     The RedFieldPosition, GreenFieldPosition, BlueFieldPosition, and
;     RsvdFieldPosition fields define the bit position within the direct
;     color pixel or YUV pixel of the least significant bit of the
;     respective color component. A color value can be aligned with its
;     pixel field by shifting the value left by the FieldPosition. For
;     example, the FieldPosition values for a Direct Color 5:6:5 mode would
;     be 11, 5, 0, and 0, for the red, green, blue, and reserved fields,
;     respectively. Note that in the YUV MemoryModel, the red field is used
;     for V, the green field is used for Y, and the blue field is used for
;     U. The FieldPosition fields should be set to 0 in modes using a memory
;     model that does not have pixels with component fields.
;
;     The DirectColorModeInfo field describes important characteristics of
;     direct color modes.  Bit D0 specifies whether the color ramp of the
;     DAC is fixed or programmable. If the color ramp is fixed, then it can
;     not be changed. If the color ramp is programmable, it is assumed that
;     the red, green, and blue lookup tables can be loaded by using VBE
;     Function 09h.  Bit D1 specifies whether the bits in the Rsvd field of
;     the direct color pixel can be used by the application or are reserved,
;     and thus unusable.
;
;     D0   =    Color ramp is fixed/programmable
;          0 =  Color ramp is fixed
;          1 =  Color ramp is programmable
;     D1   =    Bits in Rsvd field are usable/reserved
;          0 =  Bits in Rsvd field are reserved
;          1 =  Bits in Rsvd field are usable by the application
;
;     The PhysBasePtr is a 32 bit physical address of the start of frame
;     buffer memory when the controller is in flat frame buffer memory mode.
;     If this mode is not available, then this field will be zero.
;
;     The OffScreenMemOffset is a 32 bit offset from the start of the frame
;     buffer memory.  Extra off-screen memory that is needed by the
;     controller may be located either before or after this off screen
;     memory, be sure to check OffScreenMemSize to determine the amount of
;     off-screen memory which is available to the application.
;
;     The OffScreenMemSize contains the amount of available, contiguous off-
;     screen memory in 1k units, which can be used by the application.
;
;     Note: Version 1.1 and later VBE will zero out all unused fields in the
;     Mode Information Block, always returning exactly 256 bytes. This
;     facilitates upward compatibility with future versions of the standard,
;     as any newly added fields will be designed such that values of zero
;     will indicate nominal defaults or non-implementation of optional
;     features. (For example, a field containing a bit-mask of extended
;     capabilities would reflect the absence of all such capabilities.)
;     Applications that wish to be backwards compatible to Version 1.0 VBE
;     should pre-initialize the 256 byte buffer before calling the Return
;     VBE Mode Information function.

