;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
; Graph Mode Functions
;-----------------------------------------------------------------------------------

;os_g_rowcol_to_yx
;os_g_yx_to_rowcol
;os_g_set_bgfg
;os_g_get_bgfg
;os_g_get_fontinfo
;os_g_get_lfbregs
;os_g_get_yxresolution
;os_g_is_graph
;os_g_pixel
;os_g_fillrect
;os_g_saverect
;os_g_restorerect
;os_g_line
;os_g_print

; -----------------------------------------------------------------------------
; os_g_rowcol_to_yx -- Convert row/column in pixel coordinates
; IN:	AL  = row
;	AH  = column
; OUT:	RAX = y start	(pixel)
;	RBX = x start	(pixel)
os_g_rowcol_to_yx:
	push rcx
	push rdx

	xor rdx, rdx
	mov cx, ax
	mov rax, [os_System_Font_xsize]
	mul ch
	mov rbx, rax

	xor rdx, rdx
	mov rax, [os_System_Font_ysize]
	mul cl

	pop rdx
	pop rcx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_yx_to_rowcol -- Convert pixel coordinates in row/column
; IN:	RAX = y start	(pixel)
;	RBX = x start	(pixel)
; OUT:	AL  = row
;	AH  = column
os_g_yx_to_rowcol:
	push rcx
	push rdx

	xor rdx, rdx
	mov rcx, [os_System_Font_ysize]
	div rcx
	xchg rax, rbx			; rax=>x, rbx=>row

	xor rdx, rdx
	mov rcx, [os_System_Font_xsize]
	div rcx
	mov ah, al

	mov al, bl

	pop rdx
	pop rcx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_set_bgfg -- Set background-foreground colors
; IN :	RCX = color
; OUT:	All registers preserved
os_g_set_bgfg:

	mov [os_Color_bgfg], rcx

ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_get_bgfg -- Get background-foreground colors
; IN :	None
; OUT:	RCX = color
os_g_get_bgfg:

	mov rcx, [os_Color_bgfg]

ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_get_fontinfo -- Get font information
; IN :	RAX = requested info (1=font(address), 2=ysize, 3=xsize, 4=(x*y)size, 5=dispy, 6=dispx)
; OUT:  RAX = info
os_g_get_fontinfo:
	
	cmp rax, 1
	je .font
	cmp rax, 2
	je .font_y
	cmp rax, 3
	je .font_x
	cmp rax, 4
	je .font_xy
	cmp rax, 5
	je .font_dispy
	cmp rax, 6
	je .font_dispx
.font:
	mov rax, [os_System_Font]
	jmp .end
.font_y:
	mov rax, [os_System_Font_ysize]
	jmp .end
.font_x:
	mov rax, [os_System_Font_xsize]
	jmp .end
.font_xy:
	mov rax, [os_System_Font_xysize]
	jmp .end
.font_dispy:
	mov rax, [os_Disp_y]
	jmp .end
.font_dispx:
	mov rax, [os_Disp_x]

.end:

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_get_lfbregs -- Get LFB info in R14 an R15 registers
; IN :  None
; OUT:	R14 = pointer to LFB
;	R15 = bytes per line
os_g_get_lfbregs:

	xor r14, r14
	mov r14d, [vbe.LFB]
	xor r15, r15
	mov r15w, [vbe.bytesperline]

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_get_yxresolution -- Get screen resolution in pixel
; IN :  None
; OUT:	RAX = y resolution
;	RBX = x resolution
os_g_get_yxresolution:

	xor rax, rax
	xor rbx, rbx
	mov ax, [vbe.yres]
	mov bx, [vbe.xres]

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_is_graph -- Check if graph mode is on
; IN :  None
; OUT:	set carry (if carry is set = graph mode)
os_g_is_graph:

	cmp [os_GUI], 0
	jne .graph
	
	clc
	jmp .end
.graph:
	stc
.end:
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_pixel -- Draw a pixel on screen
; IN:	ECX = color			(ex.: 0x0000FF00 - argb)
; 	RAX = y
;	RBX = x
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_pixel:
	push rax
	push rbx
	push rdx

	cmp ax, [vbe.yres]
	jae .end
	cmp bx, [vbe.xres]
	jae .end

	; rax = x*4 + y*bytesxline (offset)
	shl rbx, 2
	mul r15
	add rax, rbx

	mov dword[r14+rax], ecx
.end:
	pop rdx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_fillrect -- Fill a rectangle
; IN:	ECX = color			(ex.: 0x0000FF00 - argb)
;	RAX = y start
;	RBX = x start
;	RSI = y size
; 	RDI = x size
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_fillrect:
	push rax
	push rbx
	push rdx
	push rsi
	push r8

	mov r8, rbx				; save value
.rect:	
	mov rdx, rdi				; num. pixel x line
.line:
	call os_g_pixel				; *
	inc rbx					; x (next pixel)
	dec rdx
	cmp rdx, 0
	jne .line

	mov rbx, r8				; restore value
	inc rax					; y (next line)
	dec rsi
	cmp rsi, 0
	jne .rect

.end:
	pop r8
	pop rsi
	pop rdx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_saverect -- Save a rectangle area of screen in memory
; IN:	RCX = memory location
;	RAX = y start
;	RBX = x start
;	RSI = y size
; 	RDI = x size
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_saverect:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push r8

	mov r8, rbx				; save value
.rect:	
	mov rdx, rdi				; num. pixel x line
.line:
	push rax
	push rbx
	push rdx
	shl rbx, 2
	mul r15
	add rax, rbx
	mov ebx, dword[r14+rax]
	mov dword[rcx], ebx
	add rcx, 4
	pop rdx
	pop rbx
	pop rax

	inc rbx					; x (next pixel)
	dec rdx
	cmp rdx, 0
	jne .line

	mov rbx, r8				; restore value
	inc rax					; y (next line)
	dec rsi
	cmp rsi, 0
	jne .rect

.end:
	pop r8
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_restorerect -- Copy a rectangle area from memory to screen
; IN:	RCX = memory location
;	RAX = y start
;	RBX = x start
;	RSI = y size
; 	RDI = x size
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_restorerect:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push r8

	mov r8, rbx				; save value
.rect:	
	mov rdx, rdi				; num. pixel x line
.line:
	push rax
	push rbx
	push rdx
	shl rbx, 2
	mul r15
	add rax, rbx
	mov ebx, dword[rcx]
	mov dword[r14+rax], ebx
	add rcx, 4
	pop rdx
	pop rbx
	pop rax

	inc rbx					; x (next pixel)
	dec rdx
	cmp rdx, 0
	jne .line

	mov rbx, r8				; restore value
	inc rax					; y (next line)
	dec rsi
	cmp rsi, 0
	jne .rect

.end:
	pop r8
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_line - Draw a line using the Bresenham algorithm
; IN:	ECX = color		(ex.: 0x0000FF00 - argb)
;	RAX = y1
;	RBX = x1
;	RSI = y2
; 	RDI = x2
; OUT:	All registers preserved
os_g_line:				; Bresenham algorithm
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi
	push r8 			; |x1-x2|
	push r9 			; |y1-y2|
	push r10
	push r11			; error
	push r12			; direction bit 0-> x, bit 1->y

	; clear error and direction
	mov r11, 0
	mov r12, 0

	; dxx=|x1-x2|
	mov r10, rbx
	sub r10, rdi 			; r10 = x1-x2
	; in r10 number without sign
	mov rdx, r10 			; save r10 in rdx
	neg r10 			; invert number sign
	cmp r10, rdx			; if r10<rdx then...
	jge .r1m
	mov r10, rdx			; ...place rdx in r10
	or r12b, 0x00000001		; set x direction bit 0
	.r1m:
	mov r8, r10 			; save |x1-x2| in r8

	; dyy=|y1-y2|
	mov r10, rax
	sub r10, rsi 			; r10 = y1-y2
	; in r10 number without sign
	mov rdx, r10 			; save r10 in rdx
	neg r10 			; invert number sign
	cmp r10, rdx			; if r10<rdx then...
	jge .r2m
	mov r10, rdx			; ...place rdx in r10
	or r12b, 0x00000010		; set y direction bit 1
	.r2m:
	mov r9, r10 			; save |y1-y2| in r9

	; select painting direction
	cmp r8, r10 			; |x1-x2|?|y1-y2|
	jge .h_line 			; if |x1-x2|>=|y1-y2|
	jmp .v_line 			; if |y1-y2|>|x1-x2|

;---------------------------------------
.h_line:
   	; painting always from left to right
	cmp rbx, rdi
	jg .swap_h			; if x1>x2 then interchange x and y coordinates
	jmp .start_paint_h
.swap_h: 				; x=y y=x
	xchg rax, rsi
	xchg rbx, rdi
	btc r12, 1			; reverse direction

.start_paint_h:
	mov r10, r8 			; r10=|x1-x2|  (counter)

	; start pixel y=y0 x=x0
	call os_g_pixel			; *

.loop_paint_h:
	inc rbx 			; inc x
	call os_g_pixel			; *

	mov rdx, r9
	add rdx, r11 			; error=error+|y1-y2|
	mov r11, rdx
	imul rdx, 00000002h 		; rdx=error*2
	cmp rdx, r8 			; if error*2>=|x1-x2|
	jge .inc_y
	jmp .next_step_h

.inc_y:
	test r12b, 0x00000010
	jz .incy
	dec rax				; dec y
	jmp .errh
.incy:
	inc rax 			; inc y
.errh:
	mov rdx, r11
	sub rdx, r8
	mov r11, rdx 			; error=error-|x1-x2|

.next_step_h: 
	dec r10				; decrement counter
	cmp r10, 0
	jne .loop_paint_h		; print next point
	jmp .end

;---------------------------------------
.v_line:
	; painting always from left to right
	cmp rax, rsi
	jg .swap_v			; if y1>y2 then interchange  x and y coordinates
	jmp .start_paint_v
.swap_v: 				; x=y y=x
	xchg rax, rsi
	xchg rbx, rdi
	btc r12, 0			; reverse direction

.start_paint_v:
	mov r10, r9 			; r10=|y1-y2|  (counter)

	; start pixel y=y0 x=x0
	call os_g_pixel			; *

.loop_paint_v:
	inc rax 			; inc y
	call os_g_pixel			; *

	mov rdx, r8
	add rdx, r11 			; error=error+|x1-x2|
	mov r11, rdx
	imul rdx, 00000002h 		; rdx=error*2
	cmp rdx, r9 			; if error*2>=|y1-y2|
	jge .inc_x
	jmp .next_step_v

.inc_x:
	test r12b, 0x00000001
	jz .incx
	dec rbx				; dec x
	jmp .errv
.incx:
	inc rbx 			; inc x
.errv:
	mov rdx, r11
	sub rdx, r9
	mov r11, rdx 			; error=error-|y1-y2|

.next_step_v:	
	dec r10				; decrement counter
	cmp r10, 0
	jne .loop_paint_v		; print next point

.end:
	pop r12
	pop r11
	pop r10
	pop r9
	pop r8
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_print -- Print a bitmap char using a bytes mask
; IN:	RCX = color	(Background argb - Foreground argb)
;	RAX = y start	(pixel)
;	RBX = x start	"
;	RSI = y size	"
; 	RDI = x size	"
;	R8  = address of bytes mask
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_print:
	push rax
	push rbx
	push rdx
	push rdi
	push r8
	push r14

	; Displacement
	add rax, [os_Disp_y]
	add rbx, [os_Disp_x]

	; start point
	; rax = x*4 + y*bytesxline (offset)
	shl rbx, 2
	mul r15
	add rax, rbx
	add r14, rax			; add offset

.rect:
	;--------------------------------------------------------
	mov rbx, rdi
	mov rdx, 0
.line:
	;--------------- bytes mask
	mov al, byte[r8]
	inc r8
	cmp al, 'o'
	jne .back
	;---------------
.fore:
	mov dword[r14+rdx], ecx		; argb
	jmp .next
.back:
	ror rcx, 32
	mov dword[r14+rdx], ecx		; argb
	rol rcx, 32
.next:
	add rdx, 4
	dec rbx
	cmp rbx, 0
	jne .line
	;--------------------------------------------------------

	add r14, r15			; start of next line in LFB
	dec rsi				; dec counter of lines
	cmp rsi, 0			; until last line to draw
	jne .rect 			; next line

	pop r14
	pop r8
	pop rdi
	pop rdx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; ===================================================================================================
; EOF

