;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.3 - GPL3 - Aug 2012
;
;-----------------------------------------------------------------------------------
;
; This code is based on:
; - BareMetalOS - Pure64 : http://www.returninfinity.com/baremetal.html
; - OSdev Wiki : http://wiki.osdev.org
; - Code available at FASM site : http://www.flatassembler.net
;-----------------------------------------------------------------------------------

ORG	8000h

USE16

	jmp start

;====================================================;
;  include 16-bit real mode.                         ;
;====================================================;
	include 'include16\text.inc'
	include 'include16\messages.inc'
  	include 'include16\GDT.inc'
	include 'include16\vbe.inc'
	include 'include16\a20.inc'
	include 'include16\e820.inc'
	include 'include16\check64.inc'
	include 'include16\sysvar.inc'	
	include 'include16\load_kernel.inc'
	include 'include16\trampoline.inc'		; (must be the last)


;------------------------------------------------------------------------------

USE16

start:

	cli						; Disable all interrupts
	mov [temp_byte], dl				; Save boot drive number
	xor eax, eax
	xor ebx, ebx
	xor ecx, ecx
	xor edx, edx
	xor esi, esi
	xor edi, edi
	xor ebp, ebp
	mov ds, ax
	mov es, ax
	mov ss, ax
	mov fs, ax
	mov gs, ax
	mov esp, 0x8000					; Set a known free location for the stack

	inc [cpu]
	cmp [cpu], 1
	je continue
	jmp trampoline


continue:
	sti						; Enable all interrupts
	
	mov dl, [temp_byte]
	mov [DriveNumber], dl

	; Set to 80x25 color text mode
	mov ax, 0x0003
	int 0x10
	; Hide the cursor
	mov ax, 0x0100
	mov cx, 0x200F
	int 0x10

	; Print message
	;--------------
	mov ax, 106+7
	mov bx, msg_hello
	mov ch,' '
	call RM_PutStr
	mov ax, 160+7
	mov bx, msg_s2
	mov ch,' '
	call RM_PutStr

	; Check 64-bit
	;---------------
	call RM_Check_64

	; Select here the default graphic resolution
	;--------------------------------------------   ; note: vesamodes are not standardized
	mov [VESAMODE], 277				; 800x600x32bit (default) - 0x115
	mov [IDMODE], 9					; ID of selected mode

	; VBE Check and Info
	;-------------------------
	call RM_Check_VBE				; Check

	call RM_Info_VBE				; Make table and override modes

	; Select Text Mode or Graphic Mode
	;---------------------------------
	call RM_clear_keyboard_buffer

	mov ax, 200+160+7
	mov ch, 0x24					; red on green
	mov bx, msg_textmode				; print msg
	call RM_PutStr

	mov bx, 4					; Seconds
	call RM_Wait

	mov ax, 200+160+7
	mov ch, 0x00					; black
	mov bx, msg_tmclear				; clear msg
	call RM_PutStr

	mov [GUI], 1					; Start with GUI on
	mov  ah, 01h					; Query Keyboard Status
        int  16h       		      
 	jz  .vbe				
	mov [GUI], 0					; if press any key

	cmp ah, 59 ; F1					; if F1 to...
	jb .vbe
	cmp ah, 68 ; F10				; ...F10 key pressed
	ja .vbe
	call RM_Override_VBE				; override default resolution
							; GUI=1
.vbe:
	; VBE Set
	;-------------------------
	mov al, [GUI]
	cmp al, 0
	je  .noGUI
	call RM_Init_VBE				; Set
	.noGUI:


	; Enable the A20 gate
	;--------------------
	call RM_Init_a20

	; E820 memory map
	;----------------
	call RM_BiosE820

	; Switch to Unreal mode and Load the Kernel
	;------------------------------------------
	call UM_Load_Kernel

	; Print message
	;--------------
	mov ax, 320+7
	mov bx, msg_kernel
	mov ch,' '
	call RM_PutStr

	; Prepare to jump
	;----------------
	cli						; disable the interrupts

	lgdt	[cs:GDTR]				; load GDT register

	mov	eax,cr0 				; switch to protected mode
	or	al,1
	mov	cr0,eax

	jmp	CODE_SELECTOR:pm_start


;-------------------------------------

USE32

;====================================================;
;  include 32-bit protected mode                     ;
;====================================================;
	include 'include32\splash.inc'
	include 'include32\paging.inc'
	include 'include32\CR.inc'


pm_start:

	xor	eax, eax
	mov	eax, DATA_SELECTOR			; load 4 GB data descriptor
	mov	ds, ax					; to all data segment registers
	mov	es, ax
	mov	fs, ax
	mov	gs, ax
	mov	ss, ax
	mov    esp, 0x8000				; Set a known free location for the stack

	; Splash screen
	;--------------
	mov al, [GUI]
	cmp al, 0
	je  .noSplash
	call	PM_FillBg_VBE
	.noSplash:

	; Paging
	;-------
	call PM_Init_Paging

	; PAE, CR3, LME, CR0
	;-------------------
	call PM_Init_CR


	jmp	LONG_SELECTOR:long_start

;----------------------------------------

USE64

;====================================================;
;  include 64-bit long mode                          ;
;====================================================;
	include 'include64\text64.inc'
	include 'include64\pic.inc'
	include 'include64\IDT.inc'
	include 'include64\interrupt.inc'
	include 'include64\SMP.inc'
	include 'include64\infomap.inc'

long_start:

	; Build the rest of the page tables (4GiB+)
	;------------------------------------------
	mov rcx, 0x0000000000000000
	mov rax, 0x000000010000008F
	mov rdi, PD + 8*2048
buildem:
	stosq
	add rax, 0x0000000000200000
	add rcx, 1
	cmp rcx, 30720					; Another 60 GiB (We already mapped 4 GiB)
	jne buildem
	; We have 64 GiB mapped now
	;------------------------------------------

	; Print message
	;--------------
	mov ax, 40+7
	mov bx, 4
	mov ch, ' '
	mov dx, 80
	mov rsi, msg_paging
	call LM_WriteTextString


	; Remap IRQ
	;------------
	call LM_Init_pic

	; Interrupt Descriptor Table
	;---------------------------
	call LM_Init_IDT

	sti						; now we may enable the interrupts


	; Initialize SMP
	;---------------------------
	jmp LM_Init_SMP

smp_ended:


	; Print message
	;--------------
	mov ax, 0+7
	mov bx, 5
	mov ch, ' '
	mov dx, 80
	mov rsi, msg_cores
	call LM_WriteTextString

	xor edx, edx
	mov dx, word [cpu_activated]
	mov ax, 25
	mov bx, 5
	mov ch,' '
	call LM_Write_TXT_HEX32


	; Infomap
	;---------------------------
	call LM_Init_Infomap

	; Print message
	;--------------
	mov ax, 40+7
	mov bx, 5
	mov ch, ' '
	mov dx, 80
	mov rsi, msg_64bit
	call LM_WriteTextString

	mov ax, 40+7
	mov bx, 6
	mov ch, ' '
	mov dx, 80
	mov rsi, msg_starting
	call LM_WriteTextString

	mov ax, 0					; Clear debug messages
	mov bx, 0
	mov ch, 0x07
	mov dx, 80
	mov rsi, msg_clear
	call LM_WriteTextString


	cli						; Disable Interrupt


	jmp 0x0000000000400000				; Jump to the kernel



;---------------------------------
;Variable section
;---------------------------------
; Pad to an even KB file (32 KiB)

 times 32768-($-$$) db 0x90


; =============================================================================
; EOF

;------------------------------------------------------------------------
; Memory Map:
;------------------------------------------------------------------------
;
; Start Address		End Address		Size	Description
; 0x0000000000000000	0x0000000000000FFF	4 KiB	IDT - 256 descriptors (each descriptor is 16 bytes)
; 0x0000000000001000	0x0000000000001FFF	4 KiB	Free            (pure64: GDT - 256 descriptors x 16 bytes)
; 0x0000000000002000	0x0000000000002FFF	4 KiB	PML4 - 512 entries, first entry points to PDP at 0x3000
; 0x0000000000003000	0x0000000000003FFF	4 KiB	PDP - 512 entries
;
; 0x0000000000004000	0x0000000000004FFF	4 KiB	SYS64 Data	e820 memory
; 0x0000000000005000	0x0000000000005FFF	4 KiB	SYS64 Data	Info table  (pure64: 5A00->sysvar - 5C00->vesa)
; 0x0000000000006000	0x0000000000006FFF	4 KiB	SYS64 Data	(used for temporary buffer)
; 0x0000000000007000	0x0000000000007FFF	4 KiB	SYS64 Data	(used for stack at 0x8000)
; 0x0000000000008000	0x000000000000FFFF	32 KiB	SYS64
;
; 0x0000000000010000	0x000000000001FFFF	64 KiB	Buffers
; 0x0000000000020000	0x0000000000021FFF	 8 KiB	Vesa modeinfo table
; 0x0000000000022000	0x000000000009FFFF	504 KiB	Free		
;	
; 0x00000000000A0000	0x00000000000FFFFF	384 KiB	ROM Area
; 	'' 	 	VGA mem at 0xA0000 (128 KiB) Color text starts at 0xB8000
; 	'' 	 	Video BIOS at 0xC0000 (64 KiB)
; 	'' 	 	Motherboard BIOS at F0000 (64 KiB)
;
; 0x0000000000100000	0x000000000013FFFF	256 KiB	PD - Room to map 64 GiB
; 0x0000000000140000	0x00000000001FFFFF	768 KiB Reserved for future PD
;
; 0x0000000000200000	0x00000000002FFFFF     1024 KiB AP's Stacks	(256 x 4096 bytes)
; 0x0000000000300000	0x000000000031FFFF	128 KiB Memory Map - Room to map 256 GiB with 2 MiB pages
; 0x0000000000320000	0x00000000003200FF	256 bytes CPU status data  (Bit 0 = Available, Bit 1 = Free/Busy)
; 0x0000000000320100	0x00000000003210FF	  4 KiB CPU Queue (Each queue item is 16 bytes. Room for 256 entries)
; 0x0000000000321100	0x000000000035FFFF	---	Reserved for future use
; 0x0000000000360000    0x00000000003FFFFF	---	Buffers and caches
;
; 0x0000000000400000					KERNEL
;
; 0x0000000000600000	0xFFFFFFFFFFFFFFFF	--- 	Your software is loaded here

; 
; =============================================================================================================
;
; Information table:
; SYS64 stores an information table in memory that contains various pieces of data about the computer
; before it passes control over to the kernel.
; The information table is located at 0x0000000000005000 and ends at 0x00000000000057FF (2048 bytes).
;
; Memory Address	Variable Size	Name		Description
; 0x5000		64-bit		LAPIC		Local APIC address
; 0x5008		64-bit		IOAPIC		IO APIC address
; 0x5010		16-bit		CPUSPEED	Speed of the CPUs in MegaHertz (Not Used)
; 0x5012		16-bit		CORES_ACTIVE	The number of CPU cores that were activated in the system
; 0x5014		16-bit		CORES_DETECT	The number of CPU cores that were detected in the system
; 0x5016 - 0x501F	 	 			*For future use
; 0x5020		32-bit		RAMAMOUNT	Amount of system RAM (MiB)
; 0x5024		64-bit		TABLESYSCALLS	Syscalls Table Address
; 0x502C - 0x502F	 	 			*For future use
; 0x5030		8-bit		MBR		MBR Boot Flag. Set to 1 if the disk uses MBR, 0 if not
; 0x5031		8-bit		GUI		GUI Flag. Set to 1 if graphic, 0 if text user interface
; 0x5032 - 0x503F	 	 			*For future use
; 0x5040		64-bit		ACPITABLE	Root/Extended System Description Table Address
; 0x5048 - 0x504F	 	 			*For future use
; 0x5050		64-bit		VBEINFO		Vesa Info Table Address
; 0x5058		64-bit		VBEMODE		Vesa Mode Info Table Address
; 0x5060		16-bit		VESAMODE	Current (or default) vesa mode
; 0x5062		8-bit		IDMODE		ID of selected mode
; 0x5063 - 0x56FF	 	 			*For future use
; 0x5700		8-bit		CPUFLAG0	Flags for CPU with APIC ID 0
; 0x5701		8-bit		CPUFLAG1	Flags for CPU with APIC ID 1
; 0x5702		8-bit		CPUFLAG2	Flags for CPU with APIC ID 2
; 0x5703		8-bit		CPUFLAG3	Flags for CPU with APIC ID 3
; 0x5704 - 0x57FB	 ...	 	 ...		Flags for CPU with APIC ID 4 thru to 251
; 0x57FC		8-bit		CPUFLAG252	Flags for CPU with APIC ID 252
; 0x57FD		8-bit		CPUFLAG253	Flags for CPU with APIC ID 253
; 0x57FE		8-bit		CPUFLAG254	Flags for CPU with APIC ID 254
; 0x57FF		8-bit		CPUFLAG255	Flags for CPU with APIC ID 255
;
;-------------------------------------------------------------------------------------------------------------------
;
; For the CPU Flags: Bit 0 is set if the CPU is enabled, Bit 1 is set if the CPU is the BSP.
; If Bit 0 for a CPU is not set then do not attempt to make use of it.
;
;-------------------------------------------------------------------------------------------------------------------
;
; 0x5800 - 0x5900	used for APIC Table in SMP.inc


; =================================================================================================



;	This program is free software: you can redistribute it and/or modify it under the terms
;	of the GNU General Public License as published by the Free Software Foundation,
;	either version 3 of the License, or (at your option) any later version.
;       This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
;	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
;	See the GNU General Public License for more details.

